@@ -5,9 +5,7 @@ This module is a central location for all salt exceptions
 
 # Import python libs
 import copy
-
-# Import salt libs
-import salt.utils
+import salt.exitcodes
 
 
 class SaltException(Exception):
@@ -28,6 +26,12 @@ class SaltMasterError(SaltException):
     '''
 
 
+class SaltSyndicMasterError(SaltException):
+    '''
+    Problem while proxying a request in the syndication master
+    '''
+
+
 class MasterExit(SystemExit):
     '''
     Rise when the master exits
@@ -99,6 +103,8 @@ class SaltRenderError(SaltException):
         if trace:
             exc_str += '\n{0}\n'.format(trace)
         if self.line_num and self.buffer:
+
+            import salt.utils
             self.context = salt.utils.get_context(
                 self.buffer,
                 self.line_num,
@@ -111,6 +117,17 @@ class SaltRenderError(SaltException):
         SaltException.__init__(self, exc_str)
 
 
+class SaltClientTimeout(SaltException):
+    '''
+    Thrown when a job sent through one of the Client interfaces times out
+
+    Takes the ``jid`` as a parameter
+    '''
+    def __init__(self, msg, jid=None, *args, **kwargs):
+        super(SaltClientTimeout, self).__init__(msg, *args, **kwargs)
+        self.jid = jid
+
+
 class SaltReqTimeoutError(SaltException):
     '''
     Thrown when a salt master request call fails to return within the timeout
@@ -163,3 +180,49 @@ class SaltSystemExit(SystemExit):
         SystemExit.__init__(self, code)
         if msg:
             self.message = msg
+
+
+class SaltCloudException(SaltException):
+    '''
+    Generic Salt Cloud Exception
+    '''
+
+
+class SaltCloudSystemExit(SaltCloudException):
+    '''
+    This exception is raised when the execution should be stopped.
+    '''
+    def __init__(self, message, exit_code=salt.exitcodes.EX_GENERIC):
+        SaltCloudException.__init__(self, message)
+        self.message = message
+        self.exit_code = exit_code
+
+
+class SaltCloudConfigError(SaltCloudException):
+    '''
+    Raised when a configuration setting is not found and should exist.
+    '''
+
+
+class SaltCloudNotFound(SaltCloudException):
+    '''
+    Raised when some cloud provider function cannot find what's being searched.
+    '''
+
+
+class SaltCloudExecutionTimeout(SaltCloudException):
+    '''
+    Raised when too much time has passed while querying/waiting for data.
+    '''
+
+
+class SaltCloudExecutionFailure(SaltCloudException):
+    '''
+    Raised when too much failures have occurred while querying/waiting for data.
+    '''
+
+
+class SaltCloudPasswordError(SaltCloudException):
+    '''
+    Raise when virtual terminal password input failed
+    '''
