@@ -29,6 +29,7 @@ _supported_dists += ('arch', 'mageia', 'meego', 'vmware', 'bluewhite64',
 import salt.log
 import salt.utils
 import salt.utils.network
+from salt._compat import string_types
 
 # Solve the Chicken and egg problem where grains need to run before any
 # of the modules are loaded and are generally available for any usage.
@@ -36,6 +37,7 @@ import salt.modules.cmdmod
 
 __salt__ = {
     'cmd.run': salt.modules.cmdmod._run_quiet,
+    'cmd.retcode': salt.modules.cmdmod._retcode_quiet,
     'cmd.run_all': salt.modules.cmdmod._run_all_quiet
 }
 log = logging.getLogger(__name__)
@@ -54,6 +56,8 @@ if salt.utils.is_windows():
             'will be missing'
         )
 
+_INTERFACES = {}
+
 
 def _windows_cpudata():
     '''
@@ -134,6 +138,9 @@ def _linux_gpu_data():
       - vendor: nvidia|amd|ati|...
         model: string
     '''
+    if __opts__.get('enable_gpu_grains', True) is False:
+        return {}
+
     lspci = salt.utils.which('lspci')
     if not lspci:
         log.debug(
@@ -142,15 +149,9 @@ def _linux_gpu_data():
         )
         return {}
 
-    elif __opts__.get('enable_gpu_grains', None) is False:
-        log.info(
-            'Skipping lspci call because enable_gpu_grains was set to False '
-            'in the config. GPU grains will not be available.'
-        )
-        return {}
-
     # dominant gpu vendors to search for (MUST be lowercase for matching below)
     known_vendors = ['nvidia', 'amd', 'ati', 'intel']
+    gpu_classes = ('vga compatible controller', '3d controller')
 
     devs = []
     try:
@@ -165,9 +166,8 @@ def _linux_gpu_data():
         for line in lspci_list:
             # check for record-separating empty lines
             if line == '':
-                if cur_dev.get('Class', '') == 'VGA compatible controller':
+                if cur_dev.get('Class', '').lower() in gpu_classes:
                     devs.append(cur_dev)
-                # XXX; may also need to search for "3D controller"
                 cur_dev = {}
                 continue
             if re.match(r'^\w+:\s+.*', line):
@@ -175,7 +175,7 @@ def _linux_gpu_data():
                 cur_dev[key.strip()] = val.strip()
             else:
                 error = True
-                log.debug('Unexpected lspci output: \'{0}\''.format(line))
+                log.debug('Unexpected lspci output: {0!r}'.format(line))
 
         if error:
             log.warn(
@@ -292,7 +292,7 @@ def _bsd_cpudata(osdata):
 
     grains = dict([(k, __salt__['cmd.run'](v)) for k, v in cmds.items()])
 
-    if 'cpu_flags' in grains and isinstance(grains['cpu_flags'], basestring):
+    if 'cpu_flags' in grains and isinstance(grains['cpu_flags'], string_types):
         grains['cpu_flags'] = grains['cpu_flags'].split(' ')
 
     if osdata['kernel'] == 'NetBSD':
@@ -415,7 +415,26 @@ def _virtual(osdata):
     #   virtual
     #   virtual_subtype
     grains = {'virtual': 'physical'}
-    for command in ('dmidecode', 'lspci', 'dmesg'):
+
+    # Skip the below loop on platforms which have none of the desired cmds
+    # This is a temporary measure until we can write proper virtual hardware
+    # detection.
+    skip_cmds = ('AIX',)
+
+    # Check if enable_lspci is True or False
+    if __opts__.get('enable_lspci', True) is False:
+        _cmds = ('dmidecode', 'dmesg')
+    elif osdata['kernel'] in skip_cmds:
+        _cmds = ()
+    else:
+        # /proc/bus/pci does not exists, lspci will fail
+        if not os.path.exists('/proc/bus/pci'):
+            _cmds = ('dmidecode', 'dmesg')
+        else:
+            _cmds = ('dmidecode', 'lspci', 'dmesg')
+
+    failed_commands = set()
+    for command in _cmds:
         args = []
         if osdata['kernel'] == 'Darwin':
             command = 'system_profiler'
@@ -426,7 +445,7 @@ def _virtual(osdata):
         if not cmd:
             continue
 
-        cmd = '%s %s' % (command, ' '.join(args))
+        cmd = '{0} {1}'.format(command, ' '.join(args))
 
         ret = __salt__['cmd.run_all'](cmd)
 
@@ -434,11 +453,7 @@ def _virtual(osdata):
             if salt.log.is_logging_configured():
                 if salt.utils.is_windows():
                     continue
-                log.warn(
-                    'Although \'{0}\' was found in path, the current user '
-                    'cannot execute it. Grains output might not be '
-                    'accurate.'.format(command)
-                )
+                failed_commands.add(command)
             continue
 
         output = ret['stdout']
@@ -464,6 +479,8 @@ def _virtual(osdata):
                 grains['virtual'] = 'kvm'
             if 'Vendor: Bochs' in output:
                 grains['virtual'] = 'kvm'
+            if 'BHYVE  BVXSDT' in output:
+                grains['virtual'] = 'bhyve'
             # Product Name: (oVirt) www.ovirt.org
             # Red Hat Community virtualization Project based on kvm
             elif 'Manufacturer: oVirt' in output:
@@ -488,7 +505,8 @@ def _virtual(osdata):
             model = output.lower()
             if 'vmware' in model:
                 grains['virtual'] = 'VMware'
-            # 00:04.0 System peripheral: InnoTek Systemberatung GmbH VirtualBox Guest Service
+            # 00:04.0 System peripheral: InnoTek Systemberatung GmbH
+            #         VirtualBox Guest Service
             elif 'virtualbox' in model:
                 grains['virtual'] = 'VirtualBox'
             elif 'qemu' in model:
@@ -498,12 +516,14 @@ def _virtual(osdata):
             # Break out of the loop so the next log message is not issued
             break
     else:
-        log.warn(
-            'The tools \'dmidecode\', \'lspci\' and \'dmesg\' failed to execute '
-            'because they do not exist on the system of the user running '
-            'this instance or the user does not have the necessary permissions '
-            'to execute them. Grains output might not be accurate.'
-        )
+        if osdata['kernel'] in skip_cmds:
+            log.warn(
+                'The tools \'dmidecode\', \'lspci\' and \'dmesg\' failed to '
+                'execute because they do not exist on the system of the user '
+                'running this instance or the user does not have the '
+                'necessary permissions to execute them. Grains output might '
+                'not be accurate.'
+            )
 
     choices = ('Linux', 'OpenBSD', 'HP-UX')
     isdir = os.path.isdir
@@ -511,8 +531,14 @@ def _virtual(osdata):
     if osdata['kernel'] in choices:
         if os.path.isfile('/proc/1/cgroup'):
             try:
-                if ':/lxc/' in salt.utils.fopen('/proc/1/cgroup', 'r').read():
+                if ':/lxc/' in salt.utils.fopen(
+                    '/proc/1/cgroup', 'r'
+                ).read():
                     grains['virtual_subtype'] = 'LXC'
+                if ':/docker/' in salt.utils.fopen(
+                    '/proc/1/cgroup', 'r'
+                ).read():
+                    grains['virtual_subtype'] = 'Docker'
             except IOError:
                 pass
         if isdir('/proc/vz'):
@@ -520,6 +546,9 @@ def _virtual(osdata):
                 grains['virtual'] = 'openvzhn'
             elif os.path.isfile('/proc/vz/veinfo'):
                 grains['virtual'] = 'openvzve'
+                # a posteriori, it's expected for these to have failed:
+                failed_commands.discard('lspci')
+                failed_commands.discard('dmidecode')
         # Provide additional detection for OpenVZ
         if os.path.isfile('/proc/self/status'):
             with salt.utils.fopen('/proc/self/status') as status_file:
@@ -530,7 +559,8 @@ def _virtual(osdata):
                         grains['virtual'] = 'openvzve'
                     elif vz_match and int(vz_match.groups()[0]) == 0:
                         grains['virtual'] = 'openvzhn'
-        if isdir('/proc/sys/xen') or isdir('/sys/bus/xen') or isdir('/proc/xen'):
+        if isdir('/proc/sys/xen') or \
+                isdir('/sys/bus/xen') or isdir('/proc/xen'):
             if os.path.isfile('/proc/xen/xsd_kva'):
                 # Tested on CentOS 5.3 / 2.6.18-194.26.1.el5xen
                 # Tested on CentOS 5.4 / 2.6.18-164.15.1.el5xen
@@ -539,7 +569,8 @@ def _virtual(osdata):
                 if grains.get('productname', '') == 'HVM domU':
                     # Requires dmidecode!
                     grains['virtual_subtype'] = 'Xen HVM DomU'
-                elif os.path.isfile('/proc/xen/capabilities') and os.access('/proc/xen/capabilities', os.R_OK):
+                elif os.path.isfile('/proc/xen/capabilities') and \
+                        os.access('/proc/xen/capabilities', os.R_OK):
                     caps = salt.utils.fopen('/proc/xen/capabilities')
                     if 'control_d' not in caps.read():
                         # Tested on CentOS 5.5 / 2.6.18-194.3.1.el5xen
@@ -561,21 +592,28 @@ def _virtual(osdata):
             if 'dom' in grains.get('virtual_subtype', '').lower():
                 grains['virtual'] = 'xen'
         if os.path.isfile('/proc/cpuinfo'):
-            if 'QEMU Virtual CPU' in salt.utils.fopen('/proc/cpuinfo', 'r').read():
+            if 'QEMU Virtual CPU' in \
+                    salt.utils.fopen('/proc/cpuinfo', 'r').read():
                 grains['virtual'] = 'kvm'
     elif osdata['kernel'] == 'FreeBSD':
         kenv = salt.utils.which('kenv')
         if kenv:
-            product = __salt__['cmd.run']('{0} smbios.system.product'.format(kenv))
+            product = __salt__['cmd.run'](
+                '{0} smbios.system.product'.format(kenv)
+            )
             maker = __salt__['cmd.run']('{0} smbios.system.maker'.format(kenv))
             if product.startswith('VMware'):
                 grains['virtual'] = 'VMware'
             if maker.startswith('Xen'):
                 grains['virtual_subtype'] = '{0} {1}'.format(maker, product)
                 grains['virtual'] = 'xen'
+            if maker.startswith('Microsoft') and product.startswith('Virtual'):
+                grains['virtual'] = 'VirtualPC'
         if sysctl:
             model = __salt__['cmd.run']('{0} hw.model'.format(sysctl))
-            jail = __salt__['cmd.run']('{0} -n security.jail.jailed'.format(sysctl))
+            jail = __salt__['cmd.run'](
+                '{0} -n security.jail.jailed'.format(sysctl)
+            )
             if jail == '1':
                 grains['virtual_subtype'] = 'jail'
             if 'QEMU Virtual CPU' in model:
@@ -609,6 +647,12 @@ def _virtual(osdata):
                 if os.path.isfile('/var/run/xenconsoled.pid'):
                     grains['virtual_subtype'] = 'Xen Dom0'
 
+    for command in failed_commands:
+        log.warn(
+            'Although {0!r} was found in path, the current user '
+            'cannot execute it. Grains output might not be '
+            'accurate.'.format(command)
+        )
     return grains
 
 
@@ -625,7 +669,11 @@ def _ps(osdata):
     elif osdata['os'] == 'Windows':
         grains['ps'] = 'tasklist.exe'
     elif osdata.get('virtual', '') == 'openvzhn':
-        grains['ps'] = 'ps -fH -p $(grep -l \"^envID:[[:space:]]*0\\$\" /proc/[0-9]*/status | sed -e \"s=/proc/\\([0-9]*\\)/.*=\\1=\")  | awk \'{ $7=\"\"; print }\''
+        grains['ps'] = (
+            'ps -fH -p $(grep -l \"^envID:[[:space:]]*0\\$\" '
+            '/proc/[0-9]*/status | sed -e \"s=/proc/\\([0-9]*\\)/.*=\\1=\")  '
+            '| awk \'{ $7=\"\"; print }\''
+        )
     elif osdata['os_family'] == 'Debian':
         grains['ps'] = 'ps -efHww'
     else:
@@ -771,6 +819,26 @@ _OS_FAMILY_MAP = {
 }
 
 
+def _linux_bin_exists(binary):
+    '''
+    Does a binary exist in linux (depends on which)
+    '''
+    return __salt__['cmd.retcode'](
+        'which {0}'.format(binary)
+    ) == 0
+
+
+def _get_interfaces():
+    '''
+    Provide a dict of the connected interfaces and their ip addresses
+    '''
+
+    global _INTERFACES
+    if not _INTERFACES:
+        _INTERFACES = salt.utils.network.interfaces()
+    return _INTERFACES
+
+
 def os_data():
     '''
     Return grains pertaining to the operating system
@@ -781,9 +849,12 @@ def os_data():
         }
 
     # Windows Server 2008 64-bit
-    # ('Windows', 'MINIONNAME', '2008ServerR2', '6.1.7601', 'AMD64', 'Intel64 Fam ily 6 Model 23 Stepping 6, GenuineIntel')
+    # ('Windows', 'MINIONNAME', '2008ServerR2', '6.1.7601', 'AMD64',
+    #  'Intel64 Fam ily 6 Model 23 Stepping 6, GenuineIntel')
     # Ubuntu 10.04
-    # ('Linux', 'MINIONNAME', '2.6.32-38-server', '#83-Ubuntu SMP Wed Jan 4 11:26:59 UTC 2012', 'x86_64', '')
+    # ('Linux', 'MINIONNAME', '2.6.32-38-server',
+    # '#83-Ubuntu SMP Wed Jan 4 11:26:59 UTC 2012', 'x86_64', '')
+
     # pylint: disable=unpacking-non-sequence
     (grains['kernel'], grains['nodename'],
      grains['kernelrelease'], version, grains['cpuarch'], _) = platform.uname()
@@ -800,6 +871,17 @@ def os_data():
         grains.update(_ps(grains))
         return grains
     elif salt.utils.is_linux():
+        # Add SELinux grain, if you have it
+        if _linux_bin_exists('selinuxenabled'):
+            grains['selinux'] = {}
+            grains['selinux']['enabled'] = __salt__['cmd.run'](
+                'selinuxenabled; echo $?'
+            ).strip() == '0'
+            if _linux_bin_exists('getenforce'):
+                grains['selinux']['enforced'] = __salt__['cmd.run'](
+                    'getenforce'
+                ).strip()
+
         # Add lsb grains on any distro with lsb-release
         try:
             import lsb_release
@@ -814,20 +896,26 @@ def os_data():
         except ImportError:
             # if the python library isn't available, default to regex
             if os.path.isfile('/etc/lsb-release'):
+                # Matches any possible format:
+                #     DISTRIB_ID="Ubuntu"
+                #     DISTRIB_ID='Mageia'
+                #     DISTRIB_ID=Fedora
+                #     DISTRIB_RELEASE='10.10'
+                #     DISTRIB_CODENAME='squeeze'
+                #     DISTRIB_DESCRIPTION='Ubuntu 10.10'
+                regex = re.compile((
+                    '^(DISTRIB_(?:ID|RELEASE|CODENAME|DESCRIPTION))=(?:\'|")?'
+                    '([\\w\\s\\.-_]+)(?:\'|")?'
+                ))
                 with salt.utils.fopen('/etc/lsb-release') as ifile:
                     for line in ifile:
-                        # Matches any possible format:
-                        #     DISTRIB_ID="Ubuntu"
-                        #     DISTRIB_ID='Mageia'
-                        #     DISTRIB_ID=Fedora
-                        #     DISTRIB_RELEASE='10.10'
-                        #     DISTRIB_CODENAME='squeeze'
-                        #     DISTRIB_DESCRIPTION='Ubuntu 10.10'
-                        regex = re.compile('^(DISTRIB_(?:ID|RELEASE|CODENAME|DESCRIPTION))=(?:\'|")?([\\w\\s\\.-_]+)(?:\'|")?')
                         match = regex.match(line.rstrip('\n'))
                         if match:
-                            # Adds: lsb_distrib_{id,release,codename,description}
-                            grains['lsb_{0}'.format(match.groups()[0].lower())] = match.groups()[1].rstrip()
+                            # Adds:
+                            #   lsb_distrib_{id,release,codename,description}
+                            grains[
+                                'lsb_{0}'.format(match.groups()[0].lower())
+                            ] = match.groups()[1].rstrip()
             elif os.path.isfile('/etc/os-release'):
                 # Arch ARM Linux
                 with salt.utils.fopen('/etc/os-release') as ifile:
@@ -840,8 +928,11 @@ def os_data():
                         # ANSI_COLOR="0;36"
                         # HOME_URL="http://archlinuxarm.org/"
                         # SUPPORT_URL="https://archlinuxarm.org/forum"
-                        # BUG_REPORT_URL="https://github.com/archlinuxarm/PKGBUILDs/issues"
-                        regex = re.compile('^([\\w]+)=(?:\'|")?([\\w\\s\\.-_]+)(?:\'|")?')
+                        # BUG_REPORT_URL=
+                        #   "https://github.com/archlinuxarm/PKGBUILDs/issues"
+                        regex = re.compile(
+                            '^([\\w]+)=(?:\'|")?([\\w\\s\\.-_]+)(?:\'|")?'
+                        )
                         match = regex.match(line.rstrip('\n'))
                         if match:
                             name, value = match.groups()
@@ -968,6 +1059,8 @@ def os_data():
     # architecture.
     if grains.get('os_family') == 'Debian':
         osarch = __salt__['cmd.run']('dpkg --print-architecture').strip()
+    elif grains.get('os') == 'Fedora':
+        osarch = __salt__['cmd.run']('rpm --eval %{_host_cpu}').strip()
     else:
         osarch = grains['cpuarch']
     grains['osarch'] = osarch
@@ -983,7 +1076,7 @@ def os_data():
 
     # Load additional OS family grains
     if grains['os_family'] == "RedHat":
-        grains['osmajorrelease'] = grains['osrelease'].split('.', 1)
+        grains['osmajorrelease'] = grains['osrelease'].split('.', 1)[0]
 
         grains['osfinger'] = '{os}-{ver}'.format(
             os=grains['osfullname'],
@@ -993,6 +1086,14 @@ def os_data():
             os=grains['osfullname'],
             ver=grains['osrelease'])
 
+    if grains.get('osrelease', ''):
+        osrelease_info = grains['osrelease'].split('.')
+        for idx, value in enumerate(osrelease_info):
+            if not value.isdigit():
+                continue
+            osrelease_info[idx] = int(value)
+        grains['osrelease_info'] = tuple(osrelease_info)
+
     return grains
 
 
@@ -1003,17 +1104,21 @@ def locale_info():
         defaultencoding
     '''
     grains = {}
+    grains['locale_info'] = {}
 
     if 'proxyminion' in __opts__:
         return grains
 
     try:
-        (grains['defaultlanguage'], grains['defaultencoding']) = locale.getdefaultlocale()
+        (
+            grains['locale_info']['defaultlanguage'],
+            grains['locale_info']['defaultencoding']
+        ) = locale.getdefaultlocale()
     except Exception:
         # locale.getdefaultlocale can ValueError!! Catch anything else it
         # might do, per #2205
-        grains['defaultlanguage'] = 'unknown'
-        grains['defaultencoding'] = 'unknown'
+        grains['locale_info']['defaultlanguage'] = 'unknown'
+        grains['locale_info']['defaultencoding'] = 'unknown'
     return grains
 
 
@@ -1033,10 +1138,7 @@ def hostname():
         return grains
 
     grains['localhost'] = socket.gethostname()
-    if '.' in socket.getfqdn():
-        grains['fqdn'] = socket.getfqdn()
-    else:
-        grains['fqdn'] = grains['localhost']
+    grains['fqdn'] = salt.utils.network.get_fqhostname()
     (grains['host'], grains['domain']) = grains['fqdn'].partition('.')[::2]
     return grains
 
@@ -1121,7 +1223,7 @@ def ip_interfaces():
         return {}
 
     ret = {}
-    ifaces = salt.utils.network.interfaces()
+    ifaces = _get_interfaces()
     for face in ifaces:
         iface_ips = []
         for inet in ifaces[face].get('inet', []):
@@ -1134,27 +1236,90 @@ def ip_interfaces():
     return {'ip_interfaces': ret}
 
 
+def ip4_interfaces():
+    '''
+    Provide a dict of the connected interfaces and their ip4 addresses
+    '''
+    # Provides:
+    #   ip_interfaces
+
+    if 'proxyminion' in __opts__:
+        return {}
+
+    ret = {}
+    ifaces = _get_interfaces()
+    for face in ifaces:
+        iface_ips = []
+        for inet in ifaces[face].get('inet', []):
+            if 'address' in inet:
+                iface_ips.append(inet['address'])
+        for secondary in ifaces[face].get('secondary', []):
+            if 'address' in secondary:
+                iface_ips.append(secondary['address'])
+        ret[face] = iface_ips
+    return {'ip4_interfaces': ret}
+
+
+def ip6_interfaces():
+    '''
+    Provide a dict of the connected interfaces and their ip6 addresses
+    '''
+    # Provides:
+    #   ip_interfaces
+
+    if 'proxyminion' in __opts__:
+        return {}
+
+    ret = {}
+    ifaces = _get_interfaces()
+    for face in ifaces:
+        iface_ips = []
+        for inet in ifaces[face].get('inet6', []):
+            if 'address' in inet:
+                iface_ips.append(inet['address'])
+        for secondary in ifaces[face].get('secondary', []):
+            if 'address' in secondary:
+                iface_ips.append(secondary['address'])
+        ret[face] = iface_ips
+    return {'ip6_interfaces': ret}
+
+
 def hwaddr_interfaces():
     '''
-    Provide a dict of the connected interfaces and their hw addresses (Mac Address)
+    Provide a dict of the connected interfaces and their
+    hw addresses (Mac Address)
     '''
     # Provides:
     #   hwaddr_interfaces
     ret = {}
-    ifaces = salt.utils.network.interfaces()
+    ifaces = _get_interfaces()
     for face in ifaces:
         if 'hwaddr' in ifaces[face]:
             ret[face] = ifaces[face]['hwaddr']
     return {'hwaddr_interfaces': ret}
 
 
+def get_machine_id():
+    '''
+    Provide the machine-id
+    '''
+    # Provides:
+    #   machine-id
+    locations = ['/etc/machine-id', '/var/lib/dbus/machine-id']
+    existing_locations = [loc for loc in locations if os.path.exists(loc)]
+    if not existing_locations:
+        return {}
+    else:
+        with salt.utils.fopen(existing_locations[0]) as machineid:
+            return {'machine_id': machineid.read().strip()}
+
+
 def path():
     '''
     Return the path
     '''
     # Provides:
     #   path
-
     return {'path': os.environ['PATH'].strip()}
 
 
@@ -1176,6 +1341,15 @@ def pythonpath():
     return {'pythonpath': sys.path}
 
 
+def pythonexecutable():
+    '''
+    Return the python executable in use
+    '''
+    # Provides:
+    #   pythonexecutable
+    return {'pythonexecutable': sys.executable}
+
+
 def saltpath():
     '''
     Return the path of the salt module
@@ -1202,8 +1376,11 @@ def zmqversion():
     '''
     # Provides:
     #   zmqversion
-    import zmq
-    return {'zmqversion': zmq.zmq_version()}
+    try:
+        import zmq
+        return {'zmqversion': zmq.zmq_version()}
+    except ImportError:
+        return {}
 
 
 def saltversioninfo():
@@ -1322,7 +1499,8 @@ def _hw_data(osdata):
             },
         }
         grains.update(_dmidecode_data(sunos_dmi_regex))
-    # On FreeBSD /bin/kenv (already in base system) can be used instead of dmidecode
+    # On FreeBSD /bin/kenv (already in base system)
+    # can be used instead of dmidecode
     elif osdata['kernel'] == 'FreeBSD':
         kenv = salt.utils.which('kenv')
         if kenv:
@@ -1370,6 +1548,11 @@ def _smartos_zone_data():
     # Provides:
     #   pkgsrcversion
     #   imageversion
+    #   pkgsrcpath
+    #   zonename
+    #   zoneid
+    #   hypervisor_uuid
+    #   datacenter
 
     if 'proxyminion' in __opts__:
         return {}
@@ -1378,6 +1561,7 @@ def _smartos_zone_data():
 
     pkgsrcversion = re.compile('^release:\\s(.+)')
     imageversion = re.compile('Image:\\s(.+)')
+    pkgsrcpath = re.compile('PKG_PATH=(.+)')
     if os.path.isfile('/etc/pkgsrc_version'):
         with salt.utils.fopen('/etc/pkgsrc_version', 'r') as fp_:
             for line in fp_:
@@ -1390,10 +1574,25 @@ def _smartos_zone_data():
                 match = imageversion.match(line)
                 if match:
                     grains['imageversion'] = match.group(1)
+    if os.path.isfile('/opt/local/etc/pkg_install.conf'):
+        with salt.utils.fopen('/opt/local/etc/pkg_install.conf', 'r') as fp_:
+            for line in fp_:
+                match = pkgsrcpath.match(line)
+                if match:
+                    grains['pkgsrcpath'] = match.group(1)
     if 'pkgsrcversion' not in grains:
         grains['pkgsrcversion'] = 'Unknown'
     if 'imageversion' not in grains:
         grains['imageversion'] = 'Unknown'
+    if 'pkgsrcpath' not in grains:
+        grains['pkgsrcpath'] = 'Unknown'
+
+    grains['zonename'] = __salt__['cmd.run']('zonename')
+    grains['zoneid'] = __salt__['cmd.run']('zoneadm list -p | awk -F: \'{ print $1 }\'')
+    grains['hypervisor_uuid'] = __salt__['cmd.run']('mdata-get sdc:server_uuid')
+    grains['datacenter'] = __salt__['cmd.run']('mdata-get sdc:datacenter_name')
+    if "FAILURE" in grains['datacenter'] or "No metadata" in grains['datacenter']:
+        grains['datacenter'] = "Unknown"
 
     return grains
 
@@ -1401,7 +1600,8 @@ def _smartos_zone_data():
 def get_server_id():
     '''
     Provides an integer based on the FQDN of a machine.
-    Useful as server-id in MySQL replication or anywhere else you'll need an ID like this.
+    Useful as server-id in MySQL replication or anywhere else you'll need an ID
+    like this.
     '''
     # Provides:
     #   server_id
