@@ -26,11 +26,13 @@ This module requires the ``augeas`` Python module.
 
 # Import python libs
 import os
+import re
+import logging
 
 # Make sure augeas python interface is installed
 HAS_AUGEAS = False
 try:
-    from augeas import Augeas
+    from augeas import Augeas as _Augeas
     HAS_AUGEAS = True
 except ImportError:
     pass
@@ -38,6 +40,8 @@ except ImportError:
 # Import salt libs
 from salt.exceptions import SaltInvocationError
 
+log = logging.getLogger(__name__)
+
 # Define the module's virtual name
 __virtualname__ = 'augeas'
 
@@ -54,7 +58,7 @@ def __virtual__():
 def _recurmatch(path, aug):
     '''
     Recursive generator providing the infrastructure for
-    augtools print behaviour.
+    augtools print behavior.
 
     This function is based on test_augeas.py from
     Harald Hoyer <harald@redhat.com>  in the python-augeas
@@ -81,6 +85,93 @@ def _lstrip_word(word, prefix):
     return word
 
 
+def execute(context=None, lens=None, commands=()):
+    '''
+    Execute Augeas commands
+
+    .. versionadded:: 2014.7.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' augeas.execute /files/etc/redis/redis.conf commands='["set bind 0.0.0.0", "set maxmemory 1G"]'
+    '''
+    ret = {'retval': False}
+
+    method_map = {
+        'set':    'set',
+        'mv':     'move',
+        'move':   'move',
+        'ins':    'insert',
+        'insert': 'insert',
+        'rm':     'remove',
+        'remove': 'remove',
+    }
+
+    flags = _Augeas.NO_MODL_AUTOLOAD if lens else _Augeas.NONE
+    aug = _Augeas(flags=flags)
+
+    if lens:
+        aug.add_transform(lens, re.sub('^/files', '', context))
+        aug.load()
+
+    for command in commands:
+        # first part up to space is always the command name (ie: set, move)
+        cmd, arg = command.split(' ', 1)
+        if cmd not in method_map:
+            ret['error'] = 'Command {0} is not supported (yet)'.format(cmd)
+            return ret
+
+        method = method_map[cmd]
+
+        try:
+            if method == 'set':
+                path, value, remainder = re.split('([^\'" ]+|"[^"]+"|\'[^\']+\')$', arg, 1)
+                if context:
+                    path = os.path.join(context.rstrip('/'), path.lstrip('/'))
+                value = value.strip('"').strip("'")
+                args = {'path': path, 'value': value}
+            elif method == 'move':
+                path, dst = arg.split(' ', 1)
+                if context:
+                    path = os.path.join(context.rstrip('/'), path.lstrip('/'))
+                args = {'src': path, 'dst': dst}
+            elif method == 'insert':
+                path, where, label = re.split(' (before|after) ', arg)
+                if context:
+                    path = os.path.join(context.rstrip('/'), path.lstrip('/'))
+                args = {'path': path, 'label': label, 'before': where == 'before'}
+            elif method == 'remove':
+                path = arg
+                if context:
+                    path = os.path.join(context.rstrip('/'), path.lstrip('/'))
+                args = {'path': path}
+        except ValueError as err:
+            log.error(str(err))
+            ret['error'] = 'Invalid formatted command, ' \
+                           'see debug log for details: {0}'.format(arg)
+            return ret
+
+        log.debug('{0}: {1}'.format(method, args))
+
+        func = getattr(aug, method)
+        func(**args)
+
+    try:
+        aug.save()
+        ret['retval'] = True
+    except IOError as err:
+        ret['error'] = str(err)
+
+        if lens and not lens.endswith('.lns'):
+            ret['error'] += '\nLenses are normally configured as "name.lns". ' \
+                            'Did you mean "{0}.lns"?'.format(lens)
+
+    aug.close()
+    return ret
+
+
 def get(path, value=''):
     '''
     Get a value for a specific augeas path
@@ -91,7 +182,7 @@ def get(path, value=''):
 
         salt '*' augeas.get /files/etc/hosts/1/ ipaddr
     '''
-    aug = Augeas()
+    aug = _Augeas()
     ret = {}
 
     path = path.rstrip('/')
@@ -148,7 +239,7 @@ def setvalue(*args):
 
         %wheel ALL = PASSWD : ALL , NOPASSWD : /usr/bin/apt-get , /usr/bin/aptitude
     '''
-    aug = Augeas()
+    aug = _Augeas()
     ret = {'retval': False}
 
     tuples = filter(lambda x: not str(x).startswith('prefix='), args)
@@ -192,7 +283,7 @@ def match(path, value=''):
 
         salt '*' augeas.match /files/etc/services/service-name ssh
     '''
-    aug = Augeas()
+    aug = _Augeas()
     ret = {}
 
     try:
@@ -218,7 +309,7 @@ def remove(path):
 
         salt '*' augeas.remove /files/etc/sysctl.conf/net.ipv4.conf.all.log_martians
     '''
-    aug = Augeas()
+    aug = _Augeas()
     ret = {'retval': False}
     try:
         count = aug.remove(path)
@@ -257,7 +348,7 @@ def ls(path):  # pylint: disable=C0103
             ret[_ma] = aug.get(_ma)
         return ret
 
-    aug = Augeas()
+    aug = _Augeas()
 
     path = path.rstrip('/') + '/'
     match_path = path + '*'
@@ -284,7 +375,7 @@ def tree(path):
 
         salt '*' augeas.tree /files/etc/
     '''
-    aug = Augeas()
+    aug = _Augeas()
 
     path = path.rstrip('/') + '/'
     match_path = path
