@@ -6,11 +6,12 @@ Return config information
 # Import python libs
 import re
 import os
-import urllib2
 
 # Import salt libs
 import salt.utils
+import salt._compat
 import salt.syspaths as syspaths
+import salt.utils.sdb as sdb
 
 __proxyenabled__ = ['*']
 
@@ -38,9 +39,12 @@ DEFAULTS = {'mongo.db': 'salt',
             'solr.num_backups': 1,
             'poudriere.config': '/usr/local/etc/poudriere.conf',
             'poudriere.config_dir': '/usr/local/etc/poudriere.d',
+            'ldap.uri': '',
             'ldap.server': 'localhost',
             'ldap.port': '389',
             'ldap.tls': False,
+            'ldap.no_verify': False,
+            'ldap.anonymous': True,
             'ldap.scope': 2,
             'ldap.attrs': None,
             'ldap.binddn': '',
@@ -79,8 +83,13 @@ def manage_mode(mode):
     '''
     if mode is None:
         return None
-    ret = str(mode).lstrip('0').zfill(4)
+    if not isinstance(mode, salt._compat.string_types):
+        # Make it a string in case it's not
+        mode = str(mode)
+    # Strip any quotes and initial 0, though zero-pad it up to 4
+    ret = mode.strip('"').strip('\'').lstrip('0').zfill(4)
     if ret[0] != '0':
+        # Always include a leading zero
         return '0{0}'.format(ret)
     return ret
 
@@ -122,6 +131,10 @@ def option(
             return __opts__[value]
     if not omit_master:
         if value in __pillar__.get('master', {}):
+            salt.utils.warn_until(
+                'Lithium',
+                'pillar_opts will default to False in the Lithium release'
+            )
             return __pillar__['master'][value]
     if not omit_pillar:
         if value in __pillar__:
@@ -156,6 +169,10 @@ def merge(value,
                 return ret
     if not omit_master:
         if value in __pillar__.get('master', {}):
+            salt.utils.warn_until(
+                'Lithium',
+                'pillar_opts will default to False in the Lithium release'
+            )
             tmp = __pillar__['master'][value]
             if ret is None:
                 ret = tmp
@@ -189,7 +206,7 @@ def get(key, default=''):
     '''
     .. versionadded: 0.14.0
 
-    Attempt to retrieve the named value from opts, pillar, grains of the master
+    Attempt to retrieve the named value from opts, pillar, grains or the master
     config, if the named value is not available return the passed default.
     The default return is an empty string.
 
@@ -216,18 +233,26 @@ def get(key, default=''):
 
         salt '*' config.get pkg:apache
     '''
-    ret = salt.utils.traverse_dict(__opts__, key, '_|-')
+    ret = salt.utils.traverse_dict_and_list(__opts__, key, '_|-')
     if ret != '_|-':
-        return ret
-    ret = salt.utils.traverse_dict(__grains__, key, '_|-')
+        return sdb.sdb_get(ret, __opts__)
+
+    ret = salt.utils.traverse_dict_and_list(__grains__, key, '_|-')
     if ret != '_|-':
-        return ret
-    ret = salt.utils.traverse_dict(__pillar__, key, '_|-')
+        return sdb.sdb_get(ret, __opts__)
+
+    ret = salt.utils.traverse_dict_and_list(__pillar__, key, '_|-')
     if ret != '_|-':
-        return ret
-    ret = salt.utils.traverse_dict(__pillar__.get('master', {}), key, '_|-')
+        return sdb.sdb_get(ret, __opts__)
+
+    ret = salt.utils.traverse_dict_and_list(__pillar__.get('master', {}), key, '_|-')
+    salt.utils.warn_until(
+        'Lithium',
+        'pillar_opts will default to False in the Lithium release'
+    )
     if ret != '_|-':
-        return ret
+        return sdb.sdb_get(ret, __opts__)
+
     return default
 
 
@@ -244,6 +269,10 @@ def dot_vals(value):
     '''
     ret = {}
     for key, val in __pillar__.get('master', {}).items():
+        salt.utils.warn_until(
+            'Lithium',
+            'pillar_opts will default to False in the Lithium release'
+        )
         if key.startswith('{0}.'.format(value)):
             ret[key] = val
     for key, val in __opts__.items():
