@@ -9,7 +9,6 @@ Interact with virtual machine images via libguestfs
 import os
 import tempfile
 import hashlib
-import random
 
 # Import Salt libs
 import salt.utils
@@ -48,7 +47,8 @@ def mount(location, access='rw'):
     while True:
         if os.listdir(root):
             # Stuf is in there, don't use it
-            rand = hashlib.md5(str(random.randint(1, 1000000))).hexdigest()
+            hash_type = getattr(hashlib, __opts__.get('hash_type', 'md5'))
+            rand = hash_type(os.urandom(32)).hexdigest()
             root = os.path.join(
                 tempfile.gettempdir(),
                 'guest',
