@@ -3,6 +3,9 @@
 Support for Linux LVM2
 '''
 
+# Import python libs
+import os.path
+
 # Import salt libs
 import salt.utils
 
@@ -190,6 +193,8 @@ def pvcreate(devices, **kwargs):
 
     cmd = 'pvcreate'
     for device in devices.split(','):
+        if not os.path.exists(device):
+            return '{0} does not exist'.format(device)
         cmd += ' {0}'.format(device)
     valid = ('metadatasize', 'dataalignment', 'dataalignmentoffset',
              'pvmetadatacopies', 'metadatacopies', 'metadataignore',
@@ -202,6 +207,25 @@ def pvcreate(devices, **kwargs):
     return out[0]
 
 
+def pvremove(devices):
+    '''
+    Remove a physical device being used as an LVM physical volume
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt mymachine lvm.pvremove /dev/sdb1,/dev/sdb2
+    '''
+    cmd = 'pvremove -y'
+    for device in devices.split(','):
+        if not __salt__['lvm.pvdisplay'](device):
+            return '{0} is not a physical volume'.format(device)
+        cmd += ' {0}'.format(device)
+    out = __salt__['cmd.run'](cmd).splitlines()
+    return out[0]
+
+
 def vgcreate(vgname, devices, **kwargs):
     '''
     Create an LVM volume group
@@ -230,7 +254,7 @@ def vgcreate(vgname, devices, **kwargs):
     return vgdata
 
 
-def lvcreate(lvname, vgname, size=None, extents=None, snapshot=None, pv=''):
+def lvcreate(lvname, vgname, size=None, extents=None, snapshot=None, pv='', **kwargs):
     '''
     Create a new logical volume, with option for which physical volume to be used
 
@@ -245,15 +269,41 @@ def lvcreate(lvname, vgname, size=None, extents=None, snapshot=None, pv=''):
     if size and extents:
         return 'Error: Please specify only size or extents'
 
+    valid = ('activate', 'chunksize', 'contiguous', 'discards', 'stripes',
+             'stripesize', 'minor', 'persistent', 'mirrors', 'noudevsync',
+             'monitor', 'ignoremonitoring', 'permission', 'poolmetadatasize',
+             'readahead', 'regionsize', 'thin', 'thinpool', 'type', 'virtualsize',
+             'zero',)
+    no_parameter = ('noudevsync', 'ignoremonitoring', )
+    extra_arguments = ' '.join([
+        '--{0}'.format(k) if k in no_parameter else '--{0} {1}'.format(k, v)
+        for k, v in kwargs.iteritems() if k in valid
+    ])
+
     if snapshot:
-        vgname = '-s ' + vgname + '/' + snapshot
+        _snapshot = '-s ' + vgname + '/' + snapshot
 
     if size:
-        cmd = 'lvcreate -n {0} {1} -L {2} {3}'.format(lvname, vgname, size, pv)
+        if snapshot:
+            cmd = 'lvcreate -n {0} {1} -L {2} {3}'.format(
+                lvname, _snapshot, size, pv
+            )
+        else:
+            cmd = 'lvcreate -n {0} {1} -L {2} {3}'.format(
+                lvname, vgname, size, pv
+            )
     elif extents:
-        cmd = 'lvcreate -n {0} {1} -l {2} {3}'.format(lvname, vgname, extents, pv)
+        if snapshot:
+            cmd = 'lvcreate -n {0} {1} -l {2} {3}'.format(
+                lvname, _snapshot, extents, pv
+            )
+        else:
+            cmd = 'lvcreate -n {0} {1} -l {2} {3}'.format(
+                lvname, vgname, extents, pv
+            )
     else:
         return 'Error: Either size or extents must be specified'
+
     out = __salt__['cmd.run'](cmd).splitlines()
     lvdev = '/dev/{0}/{1}'.format(vgname, lvname)
     lvdata = lvdisplay(lvdev)
