@@ -11,16 +11,16 @@ except ImportError:
 
 import salt.utils
 from salt.exceptions import CommandExecutionError, SaltInvocationError
-from salt.modules.mac_user import _osmajor, _dscl, _flush_dscl_cache
+from salt.modules.mac_user import _dscl, _flush_dscl_cache
 
 # Define the module's virtual name
 __virtualname__ = 'group'
 
 
 def __virtual__():
-    global _osmajor, _dscl, _flush_dscl_cache
-    _osmajor = salt.utils.namespaced_function(_osmajor, globals())
-    if __grains__.get('kernel') != 'Darwin' or _osmajor() < 10.7:
+    global _dscl, _flush_dscl_cache
+    if (__grains__.get('kernel') != 'Darwin' or
+            __grains__['osrelease_info'] < (10, 7)):
         return False
     _dscl = salt.utils.namespaced_function(_dscl, globals())
     _flush_dscl_cache = salt.utils.namespaced_function(
@@ -49,9 +49,15 @@ def add(name, gid=None, **kwargs):
         raise SaltInvocationError(
             'Salt will not create groups beginning with underscores'
         )
-
     if gid is not None and not isinstance(gid, int):
         raise SaltInvocationError('gid must be an integer')
+    # check if gid is already in use
+    gid_list = _list_gids()
+    if str(gid) in gid_list:
+        raise CommandExecutionError(
+            'gid {0!r} already exists'.format(gid)
+        )
+
     cmd = 'dseditgroup -o create '
     if gid:
         cmd += '-i {0} '.format(gid)
@@ -59,6 +65,19 @@ def add(name, gid=None, **kwargs):
     return __salt__['cmd.retcode'](cmd) == 0
 
 
+def _list_gids():
+    '''
+    Return a list of gids in use
+    '''
+    cmd = __salt__['cmd.run']('dscacheutil -q group | grep gid:',
+                              output_loglevel='quiet')
+    data_list = cmd.split()
+    for item in data_list:
+        if item == 'gid:':
+            data_list.remove(item)
+    return sorted(set(data_list))
+
+
 def delete(name):
     '''
     Remove the named group
