@@ -1,31 +1,45 @@
 # -*- coding: utf-8 -*-
 '''
-The Saltutil module is used to manage the state of the salt minion itself. It is used to manage minion modules as well as automate updates to the salt minion.
+The Saltutil module is used to manage the state of the salt minion itself. It
+is used to manage minion modules as well as automate updates to the salt
+minion.
 
 :depends:   - esky Python module for update functionality
 '''
 
 # Import python libs
 import os
-import hashlib
 import shutil
 import signal
 import logging
 import fnmatch
 import time
 import sys
+import copy
+from urllib2 import URLError
 
 # Import salt libs
+import salt
 import salt.payload
 import salt.state
 import salt.client
+import salt.runner
 import salt.utils
-from salt.exceptions import SaltReqTimeoutError
+import salt.utils.process
+import salt.utils.minion
+import salt.transport
+import salt.wheel
+from salt.exceptions import (
+    SaltReqTimeoutError, SaltRenderError, CommandExecutionError
+)
 from salt._compat import string_types
 
+__proxyenabled__ = ['*']
+
 # Import third party libs
 try:
     import esky
+    from esky import EskyVersionError
     HAS_ESKY = True
 except ImportError:
     HAS_ESKY = False
@@ -33,65 +47,76 @@ except ImportError:
 log = logging.getLogger(__name__)
 
 
-def _sync(form, env=None):
+def _get_top_file_envs():
+    '''
+    Get all environments from the top file
+    '''
+    try:
+        return __context__['saltutil._top_file_envs']
+    except KeyError:
+        try:
+            st_ = salt.state.HighState(__opts__)
+            top = st_.get_top()
+            if top:
+                envs = st_.top_matches(top).keys() or 'base'
+            else:
+                envs = 'base'
+        except SaltRenderError as exc:
+            raise CommandExecutionError(
+                'Unable to render top file(s): {0}'.format(exc)
+            )
+        __context__['saltutil._top_file_envs'] = envs
+        return envs
+
+
+def _sync(form, saltenv=None):
     '''
     Sync the given directory in the given environment
     '''
-    if env is None:
-        # No environment passed, detect them based on gathering the top files
-        # from the master
-        env = 'base'
-        st_ = salt.state.HighState(__opts__)
-        top = st_.get_top()
-        if top:
-            env = st_.top_matches(top).keys()
-    if isinstance(env, string_types):
-        env = env.split(',')
+    if saltenv is None:
+        saltenv = _get_top_file_envs()
+    if isinstance(saltenv, string_types):
+        saltenv = saltenv.split(',')
     ret = []
     remote = set()
     source = os.path.join('salt://_{0}'.format(form))
     mod_dir = os.path.join(__opts__['extension_modules'], '{0}'.format(form))
     if not os.path.isdir(mod_dir):
-        log.info('Creating module dir \'{0}\''.format(mod_dir))
-        os.makedirs(mod_dir)
-    for sub_env in env:
-        log.info('Syncing {0} for environment \'{1}\''.format(form, sub_env))
+        log.info('Creating module dir {0!r}'.format(mod_dir))
+        try:
+            os.makedirs(mod_dir)
+        except (IOError, OSError):
+            msg = 'Cannot create cache module directory {0}. Check permissions.'
+            log.error(msg.format(mod_dir))
+    for sub_env in saltenv:
+        log.info('Syncing {0} for environment {1!r}'.format(form, sub_env))
         cache = []
         log.info('Loading cache from {0}, for {1})'.format(source, sub_env))
-        cache.extend(__salt__['cp.cache_dir'](source, sub_env))
+        # Grab only the desired files (.py, .pyx, .so)
+        cache.extend(
+            __salt__['cp.cache_dir'](
+                source, sub_env, include_pat=r'E@\.(pyx?|so)$'
+            )
+        )
         local_cache_dir = os.path.join(
                 __opts__['cachedir'],
                 'files',
                 sub_env,
                 '_{0}'.format(form)
                 )
-        log.debug('Local cache dir: \'{0}\''.format(local_cache_dir))
+        log.debug('Local cache dir: {0!r}'.format(local_cache_dir))
         for fn_ in cache:
-            if __opts__.get('file_client', '') == 'local':
-                for fn_root in __opts__['file_roots'].get(sub_env, []):
-                    if fn_.startswith(fn_root):
-                        relpath = os.path.relpath(fn_, fn_root)
-                        relpath = relpath[relpath.index('/') + 1:]
-                        relname = os.path.splitext(relpath)[0].replace(
-                                os.sep,
-                                '.')
-                        remote.add(relpath)
-                        dest = os.path.join(mod_dir, relpath)
-            else:
-                relpath = os.path.relpath(fn_, local_cache_dir)
-                relname = os.path.splitext(relpath)[0].replace(os.sep, '.')
-                remote.add(relpath)
-                dest = os.path.join(mod_dir, relpath)
-            log.info('Copying \'{0}\' to \'{1}\''.format(fn_, dest))
+            relpath = os.path.relpath(fn_, local_cache_dir)
+            relname = os.path.splitext(relpath)[0].replace(os.sep, '.')
+            remote.add(relpath)
+            dest = os.path.join(mod_dir, relpath)
+            log.info('Copying {0!r} to {1!r}'.format(fn_, dest))
             if os.path.isfile(dest):
                 # The file is present, if the sum differs replace it
-                srch = hashlib.md5(
-                    salt.utils.fopen(fn_, 'r').read()
-                ).hexdigest()
-                dsth = hashlib.md5(
-                    salt.utils.fopen(dest, 'r').read()
-                ).hexdigest()
-                if srch != dsth:
+                hash_type = __opts__.get('hash_type', 'md5')
+                src_digest = salt.utils.get_hash(fn_, hash_type)
+                dst_digest = salt.utils.get_hash(dest, hash_type)
+                if src_digest != dst_digest:
                     # The downloaded file differs, replace!
                     shutil.copyfile(fn_, dest)
                     ret.append('{0}.{1}'.format(form, relname))
@@ -110,7 +135,7 @@ def _sync(form, env=None):
             if os.path.isfile(full):
                 touched = True
                 os.remove(full)
-        #cleanup empty dirs
+        # Cleanup empty dirs
         while True:
             emptydirs = _list_emptydirs(mod_dir)
             if not emptydirs:
@@ -118,7 +143,7 @@ def _sync(form, env=None):
             for emptydir in emptydirs:
                 touched = True
                 os.rmdir(emptydir)
-    #dest mod_dir is touched? trigger reload if requested
+    # Dest mod_dir is touched? trigger reload if requested
     if touched:
         mod_file = os.path.join(__opts__['cachedir'], 'module_refresh')
         with salt.utils.fopen(mod_file, 'a+') as ofile:
@@ -146,7 +171,11 @@ def _list_emptydirs(rootdir):
 def update(version=None):
     '''
     Update the salt minion from the URL defined in opts['update_url']
+    SaltStack, Inc provides the latest builds here:
+    update_url: http://docs.saltstack.com/downloads/
 
+    Be aware that as of 2014-8-11 there's a bug in esky such that only the
+    latest version available in the update_url can be downloaded and installed.
 
     This feature requires the minion to be running a bdist_esky build.
 
@@ -159,34 +188,53 @@ def update(version=None):
 
     .. code-block:: bash
 
+        salt '*' saltutil.update
         salt '*' saltutil.update 0.10.3
     '''
+    ret = {}
     if not HAS_ESKY:
-        return 'Esky not available as import'
+        ret['_error'] = 'Esky not available as import'
+        return ret
     if not getattr(sys, 'frozen', False):
-        return 'Minion is not running an Esky build'
+        ret['_error'] = 'Minion is not running an Esky build'
+        return ret
     if not __salt__['config.option']('update_url'):
-        return '"update_url" not configured on this minion'
+        ret['_error'] = '"update_url" not configured on this minion'
+        return ret
     app = esky.Esky(sys.executable, __opts__['update_url'])
     oldversion = __grains__['saltversion']
-    try:
-        if not version:
+    if not version:
+        try:
             version = app.find_update()
-        if not version:
-            return 'No updates available'
+        except URLError as exc:
+            ret['_error'] = 'Could not connect to update_url. Error: {0}'.format(exc)
+            return ret
+    if not version:
+        ret['_error'] = 'No updates available'
+        return ret
+    try:
         app.fetch_version(version)
+    except EskyVersionError as exc:
+        ret['_error'] = 'Unable to fetch version {0}. Error: {1}'.format(version, exc)
+        return ret
+    try:
         app.install_version(version)
+    except EskyVersionError as exc:
+        ret['_error'] = 'Unable to install version {0}. Error: {1}'.format(version, exc)
+        return ret
+    try:
         app.cleanup()
-    except Exception as err:
-        return err
+    except Exception as exc:
+        ret['_error'] = 'Unable to cleanup. Error: {0}'.format(exc)
     restarted = {}
     for service in __opts__['update_restart_services']:
         restarted[service] = __salt__['service.restart'](service)
-    return {'comment': 'Updated from {0} to {1}'.format(oldversion, version),
-            'restarted': restarted}
+    ret['comment'] = 'Updated from {0} to {1}'.format(oldversion, version)
+    ret['restarted'] = restarted
+    return ret
 
 
-def sync_modules(env=None, refresh=True):
+def sync_modules(saltenv=None, refresh=True):
     '''
     Sync the modules from the _modules directory on the salt master file
     server. This function is environment aware, pass the desired environment
@@ -199,13 +247,13 @@ def sync_modules(env=None, refresh=True):
 
         salt '*' saltutil.sync_modules
     '''
-    ret = _sync('modules', env)
+    ret = _sync('modules', saltenv)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_states(env=None, refresh=True):
+def sync_states(saltenv=None, refresh=True):
     '''
     Sync the states from the _states directory on the salt master file
     server. This function is environment aware, pass the desired environment
@@ -218,13 +266,13 @@ def sync_states(env=None, refresh=True):
 
         salt '*' saltutil.sync_states
     '''
-    ret = _sync('states', env)
+    ret = _sync('states', saltenv)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_grains(env=None, refresh=True):
+def sync_grains(saltenv=None, refresh=True):
     '''
     Sync the grains from the _grains directory on the salt master file
     server. This function is environment aware, pass the desired environment
@@ -237,14 +285,14 @@ def sync_grains(env=None, refresh=True):
 
         salt '*' saltutil.sync_grains
     '''
-    ret = _sync('grains', env)
+    ret = _sync('grains', saltenv)
     if refresh:
         refresh_modules()
         refresh_pillar()
     return ret
 
 
-def sync_renderers(env=None, refresh=True):
+def sync_renderers(saltenv=None, refresh=True):
     '''
     Sync the renderers from the _renderers directory on the salt master file
     server. This function is environment aware, pass the desired environment
@@ -257,13 +305,13 @@ def sync_renderers(env=None, refresh=True):
 
         salt '*' saltutil.sync_renderers
     '''
-    ret = _sync('renderers', env)
+    ret = _sync('renderers', saltenv)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_returners(env=None, refresh=True):
+def sync_returners(saltenv=None, refresh=True):
     '''
     Sync the returners from the _returners directory on the salt master file
     server. This function is environment aware, pass the desired environment
@@ -276,13 +324,13 @@ def sync_returners(env=None, refresh=True):
 
         salt '*' saltutil.sync_returners
     '''
-    ret = _sync('returners', env)
+    ret = _sync('returners', saltenv)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_outputters(env=None, refresh=True):
+def sync_outputters(saltenv=None, refresh=True):
     '''
     Sync the outputters from the _outputters directory on the salt master file
     server. This function is environment aware, pass the desired environment
@@ -295,13 +343,32 @@ def sync_outputters(env=None, refresh=True):
 
         salt '*' saltutil.sync_outputters
     '''
-    ret = _sync('outputters', env)
+    ret = _sync('outputters', saltenv)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_all(env=None, refresh=True):
+def sync_utils(saltenv=None, refresh=True):
+    '''
+    Sync utility source files from the _utils directory on the salt master file
+    server. This function is environment aware, pass the desired environment
+    to grab the contents of the _utils directory, base is the default
+    environment.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_utils
+    '''
+    ret = _sync('utils', saltenv)
+    if refresh:
+        refresh_modules()
+    return ret
+
+
+def sync_all(saltenv=None, refresh=True):
     '''
     Sync down all of the dynamic modules from the file server for a specific
     environment
@@ -314,12 +381,13 @@ def sync_all(env=None, refresh=True):
     '''
     log.debug('Syncing all')
     ret = {}
-    ret['modules'] = sync_modules(env, False)
-    ret['states'] = sync_states(env, False)
-    ret['grains'] = sync_grains(env, False)
-    ret['renderers'] = sync_renderers(env, False)
-    ret['returners'] = sync_returners(env, False)
-    ret['outputters'] = sync_outputters(env, False)
+    ret['modules'] = sync_modules(saltenv, False)
+    ret['states'] = sync_states(saltenv, False)
+    ret['grains'] = sync_grains(saltenv, False)
+    ret['renderers'] = sync_renderers(saltenv, False)
+    ret['returners'] = sync_returners(saltenv, False)
+    ret['outputters'] = sync_outputters(saltenv, False)
+    ret['utils'] = sync_utils(saltenv, False)
     if refresh:
         refresh_modules()
     return ret
@@ -335,7 +403,7 @@ def refresh_pillar():
 
         salt '*' saltutil.refresh_pillar
     '''
-    __salt__['event.fire']({}, 'pillar_refresh')
+    return __salt__['event.fire']({}, 'pillar_refresh')
 
 
 def refresh_modules():
@@ -348,7 +416,7 @@ def refresh_modules():
 
         salt '*' saltutil.refresh_modules
     '''
-    __salt__['event.fire']({}, 'module_refresh')
+    return __salt__['event.fire']({}, 'module_refresh')
 
 
 def is_running(fun):
@@ -380,28 +448,32 @@ def running():
 
         salt '*' saltutil.running
     '''
-    ret = []
-    serial = salt.payload.Serial(__opts__)
-    pid = os.getpid()
-    proc_dir = os.path.join(__opts__['cachedir'], 'proc')
-    if not os.path.isdir(proc_dir):
-        return []
-    for fn_ in os.listdir(proc_dir):
-        path = os.path.join(proc_dir, fn_)
-        with salt.utils.fopen(path, 'rb') as fp_:
-            data = serial.loads(fp_.read())
-        if not isinstance(data, dict):
-            # Invalid serial object
-            continue
-        if not salt.utils.process.os_is_running(data['pid']):
-            # The process is no longer running, clear out the file and
-            # continue
-            os.remove(path)
-            continue
-        if data.get('pid') == pid:
-            continue
-        ret.append(data)
-    return ret
+    return salt.utils.minion.running(__opts__)
+
+
+def clear_cache():
+    '''
+    Forcibly removes all caches on a minion.
+
+    .. versionadded:: 2014.7.0
+
+    WARNING: The safest way to clear a minion cache is by first stopping
+    the minion and then deleting the cache files before restarting it.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.clear_cache
+    '''
+    for root, dirs, files in salt.utils.safe_walk(__opts__['cachedir'], followlinks=False):
+        for name in files:
+            try:
+                os.remove(os.path.join(root, name))
+            except OSError as exc:
+                log.error('Attempt to clear cache with saltutil.clear_cache FAILED with: {0}'.format(exc))
+                return False
+    return True
 
 
 def find_job(jid):
@@ -420,6 +492,35 @@ def find_job(jid):
     return {}
 
 
+def find_cached_job(jid):
+    '''
+    Return the data for a specific cached job id
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.find_cached_job <job id>
+    '''
+    serial = salt.payload.Serial(__opts__)
+    proc_dir = os.path.join(__opts__['cachedir'], 'minion_jobs')
+    job_dir = os.path.join(proc_dir, str(jid))
+    if not os.path.isdir(job_dir):
+        return
+    path = os.path.join(job_dir, 'return.p')
+    with salt.utils.fopen(path, 'rb') as fp_:
+        buf = fp_.read()
+        fp_.close()
+        if buf:
+            data = serial.loads(buf)
+        else:
+            return
+    if not isinstance(data, dict):
+        # Invalid serial object
+        return
+    return data
+
+
 def signal_job(jid, sig):
     '''
     Sends a signal to the named salt job's process
@@ -434,6 +535,9 @@ def signal_job(jid, sig):
         if data['jid'] == jid:
             try:
                 os.kill(int(data['pid']), sig)
+                if 'child_pids' in data:
+                    for pid in data['child_pids']:
+                        os.kill(int(pid), sig)
                 return 'Signal {0} sent to job {1} at pid {2}'.format(
                         int(sig),
                         jid,
@@ -507,20 +611,46 @@ def revoke_auth():
 
         salt '*' saltutil.revoke_auth
     '''
-    sreq = salt.payload.SREQ(__opts__['master_uri'])
+    # sreq = salt.payload.SREQ(__opts__['master_uri'])
     auth = salt.crypt.SAuth(__opts__)
     tok = auth.gen_token('salt')
     load = {'cmd': 'revoke_auth',
             'id': __opts__['id'],
             'tok': tok}
+
+    sreq = salt.transport.Channel.factory(__opts__)
     try:
-        return auth.crypticle.loads(
-                sreq.send('aes', auth.crypticle.dumps(load), 1))
+        sreq.send(load)
+        # return auth.crypticle.loads(
+        #         sreq.send('aes', auth.crypticle.dumps(load), 1))
     except SaltReqTimeoutError:
         return False
     return False
 
 
+def _get_ssh_or_api_client(cfgfile, ssh=False):
+    if ssh:
+        client = salt.client.SSHClient(cfgfile)
+    else:
+        client = salt.client.get_local_client(cfgfile)
+    return client
+
+
+def _exec(client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs):
+    ret = {}
+    seen = 0
+    for ret_comp in client.cmd_iter(
+            tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs):
+        ret.update(ret_comp)
+        seen += 1
+        # ret can be empty, so we cannot len the whole return dict
+        if expr_form == 'list' and len(tgt) == seen:
+            # do not wait for timeout when explicit list matching
+            # and all results are there
+            break
+    return ret
+
+
 def cmd(tgt,
         fun,
         arg=(),
@@ -539,23 +669,22 @@ def cmd(tgt,
 
         salt '*' saltutil.cmd
     '''
-    if ssh:
-        client = salt.client.SSHClient(
-                os.path.dirname(__opts__['conf_file']))
-    else:
-        client = salt.client.LocalClient(
-                os.path.dirname(__opts__['conf_file']))
-    ret = {}
-    for ret_comp in client.cmd_iter(
-            tgt,
-            fun,
-            arg,
-            timeout,
-            expr_form,
-            ret,
-            kwarg,
-            **kwargs):
-        ret.update(ret_comp)
+    cfgfile = __opts__['conf_file']
+    client = _get_ssh_or_api_client(cfgfile, ssh)
+    ret = _exec(
+        client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
+    # if return is empty, we may have not used the right conf,
+    # try with the 'minion relative master configuration counter part
+    # if available
+    master_cfgfile = '{0}master'.format(cfgfile[:-6])  # remove 'minion'
+    if (
+        not ret
+        and cfgfile.endswith('{0}{1}'.format(os.path.sep, 'minion'))
+        and os.path.exists(master_cfgfile)
+    ):
+        client = _get_ssh_or_api_client(master_cfgfile, ssh)
+        ret = _exec(
+            client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
     return ret
 
 
@@ -578,11 +707,9 @@ def cmd_iter(tgt,
         salt '*' saltutil.cmd
     '''
     if ssh:
-        client = salt.client.SSHClient(
-                os.path.dirname(__opts__['conf_file']))
+        client = salt.client.SSHClient(__opts__['conf_file'])
     else:
-        client = salt.client.LocalClient(
-                os.path.dirname(__opts__['conf_file']))
+        client = salt.client.get_local_client(__opts__['conf_file'])
     for ret in client.cmd_iter(
             tgt,
             fun,
@@ -593,3 +720,93 @@ def cmd_iter(tgt,
             kwarg,
             **kwargs):
         yield ret
+
+
+def runner(fun, **kwargs):
+    '''
+    Execute a runner module (this function must be run on the master)
+
+    .. versionadded:: 2014.7
+
+    name
+        The name of the function to run
+    kwargs
+        Any keyword arguments to pass to the runner function
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.runner jobs.list_jobs
+    '''
+    rclient = salt.runner.RunnerClient(__opts__)
+    return rclient.cmd(fun, [], kwarg=kwargs)
+
+
+def wheel(fun, **kwargs):
+    '''
+    Execute a wheel module (this function must be run on the master)
+
+    .. versionadded:: 2014.7
+
+    name
+        The name of the function to run
+    kwargs
+        Any keyword arguments to pass to the wheel function
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.wheel key.accept match=jerry
+    '''
+    wclient = salt.wheel.WheelClient(__opts__)
+    return wclient.cmd(fun, **kwargs)
+
+
+# this is the only way I could figure out how to get the REAL file_roots
+# __opt__['file_roots'] is set to  __opt__['pillar_root']
+class _MMinion(object):
+    def __new__(cls, saltenv, reload_env=False):
+        # this is to break out of salt.loaded.int and make this a true singleton
+        # hack until https://github.com/saltstack/salt/pull/10273 is resolved
+        # this is starting to look like PHP
+        global _mminions  # pylint: disable=W0601
+        if '_mminions' not in globals():
+            _mminions = {}
+        if saltenv not in _mminions or reload_env:
+            opts = copy.deepcopy(__opts__)
+            del opts['file_roots']
+            # grains at this point are in the context of the minion
+            global __grains__  # pylint: disable=W0601
+            grains = copy.deepcopy(__grains__)
+            m = salt.minion.MasterMinion(opts)
+
+            # this assignment is so that the rest of fxns called by salt still
+            # have minion context
+            __grains__ = grains
+
+            # this assignment is so that fxns called by mminion have minion
+            # context
+            m.opts['grains'] = grains
+
+            env_roots = m.opts['file_roots'][saltenv]
+            m.opts['module_dirs'] = [fp + '/_modules' for fp in env_roots]
+            m.gen_modules()
+            _mminions[saltenv] = m
+        return _mminions[saltenv]
+
+
+def mmodule(saltenv, fun, *args, **kwargs):
+    '''
+    Loads minion modules from an environment so that they can be used in pillars
+    for that environment
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.mmodule base test.ping
+    '''
+    mminion = _MMinion(saltenv)
+    return mminion.functions[fun](*args, **kwargs)
