@@ -89,8 +89,8 @@ def available(name):
     Returns ``True`` if the specified service is available, otherwise returns
     ``False``.
 
-    We look up the name with the svcs command to get back the FMRI
-    This allows users to use simpler service names
+    The Solaris and SmartOS "if" statement uses svcs to return the service name from the
+    package name.
 
     CLI Example:
 
@@ -98,9 +98,12 @@ def available(name):
 
         salt '*' service.available net-snmp
     '''
-    cmd = '/usr/bin/svcs -H -o FMRI {0}'.format(name)
-    name = __salt__['cmd.run'](cmd)
-    return name in get_all()
+    if 'SmartOS' in __grains__['os'] or 'Solaris' in __grains__['os']:
+        cmd = '/usr/bin/svcs -H -o FMRI {0}'.format(name)
+        name = __salt__['cmd.run'](cmd)
+        return name in get_all()
+    else:
+        return name in get_all()
 
 
 def missing(name):
@@ -115,9 +118,12 @@ def missing(name):
 
         salt '*' service.missing net-snmp
     '''
-    cmd = '/usr/bin/svcs -H -o FMRI {0}'.format(name)
-    name = __salt__['cmd.run'](cmd)
-    return name not in get_all()
+    if 'SmartOS' in __grains__['os'] or 'Solaris' in __grains__['os']:
+        cmd = '/usr/bin/svcs -H -o FMRI {0}'.format(name)
+        name = __salt__['cmd.run'](cmd)
+        return name not in get_all()
+    else:
+        return name not in get_all()
 
 
 def get_all():
