@@ -15,6 +15,7 @@ import datetime
 import tempfile
 
 # Import salt libs
+import salt.config
 import salt.utils
 import salt.state
 import salt.payload
@@ -26,9 +27,13 @@ __proxyenabled__ = ['*']
 
 __outputter__ = {
     'sls': 'highstate',
+    'sls_id': 'highstate',
+    'pkg': 'highstate',
     'top': 'highstate',
     'single': 'highstate',
     'highstate': 'highstate',
+    'template': 'highstate',
+    'template_str': 'highstate',
 }
 
 log = logging.getLogger(__name__)
@@ -78,7 +83,7 @@ def _wait(jid):
         states = _prior_running_states(jid)
 
 
-def running():
+def running(concurrent=False):
     '''
     Return a dict of state return data if a state function is already running.
     This function is used to prevent multiple state calls from being run at
@@ -91,6 +96,8 @@ def running():
         salt '*' state.running
     '''
     ret = []
+    if concurrent:
+        return ret
     active = __salt__['saltutil.is_running']('state.*')
     for data in active:
         err = (
@@ -120,6 +127,30 @@ def _prior_running_states(jid):
     return ret
 
 
+def _check_queue(queue, kwargs):
+    '''
+    Utility function to queue the state run if requested
+    and to check for conflicts in currently running states
+    '''
+    if queue:
+        _wait(kwargs.get('__pub_jid'))
+    else:
+        conflict = running()
+        if conflict:
+            __context__['retcode'] = 1
+            return conflict
+
+
+def _get_opts(localconfig=None):
+    '''
+    Return a copy of the opts for use, optionally load a local config on top
+    '''
+    opts = copy.deepcopy(__opts__)
+    if localconfig:
+        opts = salt.config.minion_config(localconfig, defaults=opts)
+    return opts
+
+
 def low(data, queue=False, **kwargs):
     '''
     Execute a single low data call
@@ -131,13 +162,9 @@ def low(data, queue=False, **kwargs):
 
         salt '*' state.low '{"state": "pkg", "fun": "installed", "name": "vi"}'
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
     st_ = salt.state.State(__opts__)
     err = st_.verify_data(data)
     if err:
@@ -162,13 +189,9 @@ def high(data, queue=False, **kwargs):
 
         salt '*' state.high '{"vim": {"pkg": ["installed"]}}'
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
     st_ = salt.state.State(__opts__)
     ret = st_.call_high(data)
     _set_retcode(ret)
@@ -177,7 +200,10 @@ def high(data, queue=False, **kwargs):
 
 def template(tem, queue=False, **kwargs):
     '''
-    Execute the information stored in a template file on the minion
+    Execute the information stored in a template file on the minion.
+
+    This function does not ask a master for a SLS file to render but
+    instead directly processes the file at the provided path on the minion.
 
     CLI Example:
 
@@ -185,15 +211,17 @@ def template(tem, queue=False, **kwargs):
 
         salt '*' state.template '<Path to template on the minion>'
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
-    st_ = salt.state.State(__opts__)
-    ret = st_.call_template(tem)
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
+    st_ = salt.state.HighState(__opts__)
+    if not tem.endswith('.sls'):
+        tem = '{sls}.sls'.format(sls=tem)
+    high, errors = st_.render_state(tem, None, '', None, local=True)
+    if errors:
+        __context__['retcode'] = 1
+        return errors
+    ret = st_.state.call_high(high)
     _set_retcode(ret)
     return ret
 
@@ -208,46 +236,67 @@ def template_str(tem, queue=False, **kwargs):
 
         salt '*' state.template_str '<Template String>'
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
     st_ = salt.state.State(__opts__)
     ret = st_.call_template_str(tem)
     _set_retcode(ret)
     return ret
 
 
-def highstate(test=None, queue=False, **kwargs):
+def highstate(test=None,
+              queue=False,
+              **kwargs):
     '''
     Retrieve the state data from the salt master for this minion and execute it
 
+    test
+        Notify states to execute in test-only (dry-run) mode.
+
+        Sets the ``test`` variable in the minion ``opts`` for the duration of
+        the state run.
+    pillar
+        Custom Pillar data can be passed with the ``pillar`` kwarg. Values
+        passed here will override hard-coded Pillar values.
+    queue : ``False``
+        Instead of failing immediately when another state run is in progress,
+        queue the new state run to begin running once the other has finished.
+
+        This option starts a new thread for each queued state run so use this
+        option sparingly.
+    localconfig:
+        Instead of using running minion opts, load ``localconfig`` and merge that
+        with the running minion opts. This functionality is intended for using
+        "roots" of salt directories (with their own minion config, pillars,
+        file_roots) to run highstate out of.
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' state.highstate
 
+        salt '*' state.highstate whitelist=sls1_to_run,sls2_to_run
         salt '*' state.highstate exclude=sls_to_exclude
         salt '*' state.highstate exclude="[{'id': 'id_to_exclude'}, {'sls': 'sls_to_exclude'}]"
-    '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
-    orig_test = __opts__.get('test', None)
-    opts = copy.deepcopy(__opts__)
 
-    if salt.utils.test_mode(test=test, **kwargs):
-        opts['test'] = True
+        salt '*' state.highstate pillar="{foo: 'Foo!', bar: 'Bar!'}"
+    '''
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
+    orig_test = __opts__.get('test', None)
+
+    opts = _get_opts(kwargs.get('localconfig'))
+
+    if test is None:
+        if salt.utils.test_mode(test=test, **kwargs):
+            opts['test'] = True
+        else:
+            opts['test'] = __opts__.get('test', None)
     else:
-        opts['test'] = __opts__.get('test', None)
+        opts['test'] = test
 
     if 'env' in kwargs:
         salt.utils.warn_until(
@@ -272,7 +321,8 @@ def highstate(test=None, queue=False, **kwargs):
                 exclude=kwargs.get('exclude', []),
                 cache=kwargs.get('cache', None),
                 cache_name=kwargs.get('cache_name', 'highstate'),
-                force=kwargs.get('force', False)
+                force=kwargs.get('force', False),
+                whitelist=kwargs.get('whitelist')
                 )
     finally:
         st_.pop_active()
@@ -311,9 +361,36 @@ def sls(mods,
         env=None,
         **kwargs):
     '''
-    Execute a set list of state modules from an environment. The default
-    environment is ``base``, use ``saltenv`` (``env`` in Salt 0.17.x and older)
-    to specify a different environment
+    Execute a set list of state files from an environment.
+
+    test
+        Notify states to execute in test-only (dry-run) mode.
+
+        Sets the ``test`` variable in the minion ``opts`` for the duration of
+        the state run.
+    pillar
+        Custom Pillar data can be passed with the ``pillar`` kwarg. Values
+        passed here will override hard-coded Pillar values.
+    queue : ``False``
+        Instead of failing immediately when another state run is in progress,
+        queue the new state run to begin running once the other has finished.
+
+        This option starts a new thread for each queued state run so use this
+        option sparingly.
+    saltenv : base
+        Specify a ``file_roots`` environment.
+
+        .. versionchanged:: 0.17.0
+            Argument name changed from ``env`` to ``saltenv``.
+    concurrent:
+        WARNING: This flag is potentially dangerous. It is designed
+        for use when multiple state runs can safely be run at the same
+        Do not use this flag for performance optimization.
+    localconfig:
+        Instead of using running minion opts, load ``localconfig`` and merge that
+        with the running minion opts. This functionality is intended for using
+        "roots" of salt directories (with their own minion config, pillars,
+        file_roots) to run highstate out of.
 
     CLI Example:
 
@@ -321,7 +398,10 @@ def sls(mods,
 
         salt '*' state.sls core,edit.vim dev
         salt '*' state.sls core exclude="[{'id': 'id_to_exclude'}, {'sls': 'sls_to_exclude'}]"
+
+        salt '*' state.sls myslsfile pillar="{foo: 'Foo!', bar: 'Bar!'}"
     '''
+    concurrent = kwargs.get('concurrent', False)
     if env is not None:
         salt.utils.warn_until(
             'Boron',
@@ -334,7 +414,7 @@ def sls(mods,
     if queue:
         _wait(kwargs.get('__pub_jid'))
     else:
-        conflict = running()
+        conflict = running(concurrent)
         if conflict:
             __context__['retcode'] = 1
             return conflict
@@ -344,7 +424,7 @@ def sls(mods,
         err += __pillar__['_errors']
         return err
     orig_test = __opts__.get('test', None)
-    opts = copy.deepcopy(__opts__)
+    opts = _get_opts(kwargs.get('localconfig'))
 
     if salt.utils.test_mode(test=test, **kwargs):
         opts['test'] = True
@@ -405,8 +485,9 @@ def sls(mods,
         with salt.utils.fopen(cache_file, 'w+b') as fp_:
             serial.dump(ret, fp_)
     except (IOError, OSError):
-        msg = 'Unable to write to "state.sls" cache file {0}'
+        msg = 'Unable to write to SLS cache file {0}. Check permission.'
         log.error(msg.format(cache_file))
+
     os.umask(cumask)
     _set_retcode(ret)
     # Work around Windows multiprocessing bug, set __opts__['test'] back to
@@ -425,7 +506,10 @@ def sls(mods,
     return ret
 
 
-def top(topfn, test=None, queue=False, **kwargs):
+def top(topfn,
+        test=None,
+        queue=False,
+        **kwargs):
     '''
     Execute a specific top file instead of the default
 
@@ -437,20 +521,16 @@ def top(topfn, test=None, queue=False, **kwargs):
         salt '*' state.top reverse_top.sls exclude=sls_to_exclude
         salt '*' state.top reverse_top.sls exclude="[{'id': 'id_to_exclude'}, {'sls': 'sls_to_exclude'}]"
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
     if not _check_pillar(kwargs):
         __context__['retcode'] = 5
         err = ['Pillar failed to render with the following messages:']
         err += __pillar__['_errors']
         return err
     orig_test = __opts__.get('test', None)
-    opts = copy.deepcopy(__opts__)
+    opts = _get_opts(kwargs.get('localconfig'))
     if salt.utils.test_mode(test=test, **kwargs):
         opts['test'] = True
     else:
@@ -484,20 +564,24 @@ def show_highstate(queue=False, **kwargs):
     '''
     Retrieve the highstate data from the salt master and display it
 
+    Custom Pillar data can be passed with the ``pillar`` kwarg.
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' state.show_highstate
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
-    st_ = salt.state.HighState(__opts__)
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
+    pillar = kwargs.get('pillar')
+    if pillar is not None and not isinstance(pillar, dict):
+        raise SaltInvocationError(
+            'Pillar data must be formatted as a dictionary'
+        )
+
+    st_ = salt.state.HighState(__opts__, pillar)
     st_.push_active()
     try:
         ret = st_.compile_highstate()
@@ -518,13 +602,10 @@ def show_lowstate(queue=False, **kwargs):
 
         salt '*' state.show_lowstate
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        assert False
+        return conflict
     st_ = salt.state.HighState(__opts__)
     st_.push_active()
     try:
@@ -534,6 +615,61 @@ def show_lowstate(queue=False, **kwargs):
     return ret
 
 
+def sls_id(
+        id_,
+        mods,
+        saltenv='base',
+        test=None,
+        queue=False,
+        **kwargs):
+    '''
+    Call a single ID from the named module(s) and handle all requisites
+
+    .. versionadded:: 2014.7.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' state.sls_id apache http
+    '''
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
+    orig_test = __opts__.get('test', None)
+    opts = _get_opts(kwargs.get('localconfig'))
+    if salt.utils.test_mode(test=test, **kwargs):
+        opts['test'] = True
+    else:
+        opts['test'] = __opts__.get('test', None)
+    st_ = salt.state.HighState(opts)
+    if isinstance(mods, string_types):
+        split_mods = mods.split(',')
+    st_.push_active()
+    try:
+        high_, errors = st_.render_highstate({saltenv: split_mods})
+    finally:
+        st_.pop_active()
+    errors += st_.state.verify_high(high_)
+    if errors:
+        __context__['retcode'] = 1
+        return errors
+    chunks = st_.state.compile_high_data(high_)
+    ret = {}
+    for chunk in chunks:
+        if chunk.get('__id__', '') == id_:
+            ret.update(st_.state.call_chunk(chunk, {}, chunks))
+    # Work around Windows multiprocessing bug, set __opts__['test'] back to
+    # value from before this function was run.
+    __opts__['test'] = orig_test
+    if not ret:
+        raise SaltInvocationError(
+            'No matches for ID \'{0}\' found in SLS \'{1}\' within saltenv '
+            '\'{2}\''.format(id_, mods, saltenv)
+        )
+    return ret
+
+
 def show_low_sls(mods,
                  saltenv='base',
                  test=None,
@@ -559,16 +695,11 @@ def show_low_sls(mods,
         )
         # Backwards compatibility
         saltenv = env
-
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
     orig_test = __opts__.get('test', None)
-    opts = copy.deepcopy(__opts__)
+    opts = _get_opts(kwargs.get('localconfig'))
     if salt.utils.test_mode(test=test, **kwargs):
         opts['test'] = True
     else:
@@ -601,6 +732,8 @@ def show_sls(mods, saltenv='base', test=None, queue=False, env=None, **kwargs):
     This function does not support topfiles.  For ``top.sls`` please use
     ``show_top`` instead.
 
+    Custom Pillar data can be passed with the ``pillar`` kwarg.
+
     CLI Example:
 
     .. code-block:: bash
@@ -615,20 +748,24 @@ def show_sls(mods, saltenv='base', test=None, queue=False, env=None, **kwargs):
         )
         # Backwards compatibility
         saltenv = env
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
     orig_test = __opts__.get('test', None)
-    opts = copy.deepcopy(__opts__)
+    opts = _get_opts(kwargs.get('localconfig'))
+
     if salt.utils.test_mode(test=test, **kwargs):
         opts['test'] = True
     else:
         opts['test'] = __opts__.get('test', None)
-    st_ = salt.state.HighState(opts)
+
+    pillar = kwargs.get('pillar')
+    if pillar is not None and not isinstance(pillar, dict):
+        raise SaltInvocationError(
+            'Pillar data must be formatted as a dictionary'
+        )
+
+    st_ = salt.state.HighState(opts, pillar)
     if isinstance(mods, string_types):
         mods = mods.split(',')
     st_.push_active()
@@ -656,13 +793,9 @@ def show_top(queue=False, **kwargs):
 
         salt '*' state.show_top
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
     st_ = salt.state.HighState(__opts__)
     errors = []
     top_ = st_.get_top()
@@ -691,13 +824,9 @@ def single(fun, name, test=None, queue=False, **kwargs):
         salt '*' state.single pkg.installed name=vim
 
     '''
-    if queue:
-        _wait(kwargs.get('__pub_jid'))
-    else:
-        conflict = running()
-        if conflict:
-            __context__['retcode'] = 1
-            return conflict
+    conflict = _check_queue(queue, kwargs)
+    if conflict is not None:
+        return conflict
     comps = fun.split('.')
     if len(comps) < 2:
         __context__['retcode'] = 1
@@ -707,7 +836,7 @@ def single(fun, name, test=None, queue=False, **kwargs):
                    '__id__': name,
                    'name': name})
     orig_test = __opts__.get('test', None)
-    opts = copy.deepcopy(__opts__)
+    opts = _get_opts(kwargs.get('localconfig'))
     if salt.utils.test_mode(test=test, **kwargs):
         opts['test'] = True
     else:
@@ -791,19 +920,30 @@ def pkg(pkg_path, pkg_sum, hash_type, test=False, **kwargs):
     lowstate_json = os.path.join(root, 'lowstate.json')
     with salt.utils.fopen(lowstate_json, 'r') as fp_:
         lowstate = json.load(fp_, object_hook=salt.utils.decode_dict)
-    popts = copy.deepcopy(__opts__)
+    # Check for errors in the lowstate
+    for chunk in lowstate:
+        if not isinstance(chunk, dict):
+            return lowstate
+    pillar_json = os.path.join(root, 'pillar.json')
+    if os.path.isfile(pillar_json):
+        with salt.utils.fopen(pillar_json, 'r') as fp_:
+            pillar = json.load(fp_)
+    else:
+        pillar = None
+    popts = _get_opts(kwargs.get('localconfig'))
     popts['fileclient'] = 'local'
     popts['file_roots'] = {}
     if salt.utils.test_mode(test=test, **kwargs):
         popts['test'] = True
     else:
         popts['test'] = __opts__.get('test', None)
-    for fn_ in os.listdir(root):
+    envs = os.listdir(root)
+    for fn_ in envs:
         full = os.path.join(root, fn_)
         if not os.path.isdir(full):
             continue
         popts['file_roots'][fn_] = [full]
-    st_ = salt.state.State(popts)
+    st_ = salt.state.State(popts, pillar=pillar)
     ret = st_.call_chunks(lowstate)
     try:
         shutil.rmtree(root)
