@@ -1,6 +1,16 @@
 # -*- coding: utf-8 -*-
 '''
 Support for YUM
+
+.. note::
+    This module makes heavy use of the **repoquery** utility, from the
+    yum-utils_ package. This package will be installed as a dependency if salt
+    is installed via EPEL. However, if salt has been installed using pip, or a
+    host is being managed using salt-ssh, then as of version 2014.7.0
+    yum-utils_ will be installed automatically to satisfy this dependency.
+
+    .. _yum-utils: http://yum.baseurl.org/wiki/YumUtils
+
 '''
 
 # Import python libs
@@ -8,6 +18,7 @@ import copy
 import logging
 import os
 import re
+from distutils.version import LooseVersion as _LooseVersion
 
 # Import salt libs
 import salt.utils
@@ -101,14 +112,31 @@ def _repoquery_pkginfo(repoquery_args):
     return ret
 
 
+def _check_repoquery():
+    '''
+    Check for existence of repoquery and install yum-utils if it is not
+    present.
+    '''
+    if not salt.utils.which('repoquery'):
+        __salt__['cmd.run'](
+            ['yum', '-y', 'install', 'yum-utils'],
+            python_shell=False,
+            output_loglevel='trace'
+        )
+        # Check again now that we've installed yum-utils
+        if not salt.utils.which('repoquery'):
+            raise CommandExecutionError('Unable to install yum-utils')
+
+
 def _repoquery(repoquery_args, query_format=__QUERYFORMAT):
     '''
     Runs a repoquery command and returns a list of namedtuples
     '''
+    _check_repoquery()
     cmd = 'repoquery --plugins --queryformat="{0}" {1}'.format(
         query_format, repoquery_args
     )
-    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='debug')
+    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
     return out.splitlines()
 
 
@@ -136,13 +164,43 @@ def _get_repo_options(**kwargs):
         repo_arg = ''
         if disablerepo:
             log.info('Disabling repo {0!r}'.format(disablerepo))
-            repo_arg += '--disablerepo={0!r} '.format(disablerepo)
+            repo_arg += '--disablerepo={0!r}'.format(disablerepo)
         if enablerepo:
             log.info('Enabling repo {0!r}'.format(enablerepo))
-            repo_arg += '--enablerepo={0!r} '.format(enablerepo)
+            repo_arg += '--enablerepo={0!r}'.format(enablerepo)
     return repo_arg
 
 
+def _get_excludes_option(**kwargs):
+    '''
+    Returns a string of '--disableexcludes' option to be used in the yum command,
+    based on the kwargs.
+    '''
+    disable_excludes_arg = ''
+    disable_excludes = kwargs.get('disableexcludes', '')
+
+    if disable_excludes:
+        log.info('Disabling excludes for {0!r}'.format(disable_excludes))
+        disable_excludes_arg = ('--disableexcludes={0!r}'.format(disable_excludes))
+
+    return disable_excludes_arg
+
+
+def _get_branch_option(**kwargs):
+    '''
+    Returns a string of '--branch' option to be used in the yum command,
+    based on the kwargs. This feature requires 'branch' plugin for YUM.
+    '''
+    # Get branch option from the kwargs
+    branch = kwargs.get('branch', '')
+
+    branch_arg = ''
+    if branch:
+        log.info('Adding branch {0!r}'.format(branch))
+        branch_arg = ('--branch={0!r}'.format(branch))
+    return branch_arg
+
+
 def _check_32(arch):
     '''
     Returns True if both the OS arch and the passed arch are 32-bit
@@ -150,10 +208,37 @@ def _check_32(arch):
     return all(x in __ARCHES_32 for x in (__grains__['osarch'], arch))
 
 
+def _rpm_pkginfo(name):
+    '''
+    Parses RPM metadata and returns a pkginfo namedtuple
+    '''
+    # REPOID is not a valid tag for the rpm command. Remove it and replace it
+    # with "none"
+    queryformat = __QUERYFORMAT.replace('%{REPOID}', 'none')
+    output = __salt__['cmd.run_stdout'](
+        'rpm -qp --queryformat {0!r} {1}'.format(queryformat, name),
+        output_loglevel='trace',
+        ignore_retcode=True
+    )
+    return _parse_pkginfo(output)
+
+
+def _rpm_installed(name):
+    '''
+    Parses RPM metadata to determine if the RPM target is already installed.
+    Returns the name of the installed package if found, otherwise None.
+    '''
+    pkg = _rpm_pkginfo(name)
+    try:
+        return pkg.name if pkg.name in list_pkgs() else None
+    except AttributeError:
+        return None
+
+
 def normalize_name(name):
     '''
     Strips the architecture from the specified package name, if necessary.
-    Circomstances where this would be done include:
+    Circumstances where this would be done include:
 
     * If the arch is 32 bit and the package name ends in a 32-bit arch.
     * If the arch matches the OS arch, or is ``noarch``.
@@ -184,7 +269,11 @@ def latest_version(*names, **kwargs):
     If the latest version of a given package is already installed, an empty
     string will be returned for that package.
 
-    A specific repo can be requested using the ``fromrepo`` keyword argument.
+    A specific repo can be requested using the ``fromrepo`` keyword argument,
+    and the ``disableexcludes`` option is also supported.
+
+    .. versionadded:: 2014.7.0
+        Support for the ``disableexcludes`` option
 
     CLI Example:
 
@@ -192,6 +281,7 @@ def latest_version(*names, **kwargs):
 
         salt '*' pkg.latest_version <package name>
         salt '*' pkg.latest_version <package name> fromrepo=epel-testing
+        salt '*' pkg.latest_version <package name> disableexcludes=main
         salt '*' pkg.latest_version <package1> <package2> <package3> ...
     '''
     refresh = salt.utils.is_true(kwargs.pop('refresh', True))
@@ -227,13 +317,14 @@ def latest_version(*names, **kwargs):
 
     # Refresh before looking for the latest version available
     if refresh:
-        refresh_db()
+        refresh_db(**kwargs)
 
     # Get updates for specified package(s)
     repo_arg = _get_repo_options(**kwargs)
+    exclude_arg = _get_excludes_option(**kwargs)
     updates = _repoquery_pkginfo(
-        '{0} --pkgnarrow=available {1}'
-        .format(repo_arg, ' '.join(names))
+        '{0} {1} --pkgnarrow=available {2}'
+        .format(repo_arg, exclude_arg, ' '.join(names))
     )
 
     for name in names:
@@ -323,23 +414,42 @@ def list_pkgs(versions_as_list=False, **kwargs):
 
 def list_repo_pkgs(*args, **kwargs):
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.1.0
+    .. versionchanged:: 2014.7.0
+        All available versions of each package are now returned. This required
+        a slight modification to the structure of the return dict. The return
+        data shown below reflects the updated return dict structure.
 
-    Returns all available packages. Optionally, package names can be passed and
-    the results will be filtered to packages matching those names. This can be
-    helpful in discovering the version or repo to specify in a pkg.installed
-    state. The return data is a dictionary of repo names, with each repo having
-    a list of dictionaries denoting the package name and version. An example of
-    the return data would look like this:
+    Returns all available packages. Optionally, package names (and name globs)
+    can be passed and the results will be filtered to packages matching those
+    names. This is recommended as it speeds up the function considerably.
+
+    This function can be helpful in discovering the version or repo to specify
+    in a :mod:`pkg.installed <salt.states.pkg.installed>` state.
+
+    The return data is a dictionary of repo names, with each repo containing a
+    dictionary in which the keys are package names, and the values are a list
+    of version numbers. Here is an example of the return data:
 
     .. code-block:: python
 
         {
-            '<repo_name>': [
-                {'<package1>': '<version1>'},
-                {'<package2>': '<version2>'},
-                {'<package3>': '<version3>'}
-            ]
+            'base': {
+                'bash': ['4.1.2-15.el6_4'],
+                'kernel': ['2.6.32-431.el6']
+            },
+            'updates': {
+                'bash': ['4.1.2-15.el6_5.2', '4.1.2-15.el6_5.1'],
+                'kernel': ['2.6.32-431.29.2.el6',
+                           '2.6.32-431.23.3.el6',
+                           '2.6.32-431.20.5.el6',
+                           '2.6.32-431.20.3.el6',
+                           '2.6.32-431.17.1.el6',
+                           '2.6.32-431.11.2.el6',
+                           '2.6.32-431.5.1.el6',
+                           '2.6.32-431.3.1.el6',
+                           '2.6.32-431.1.2.0.1.el6']
+            }
         }
 
     fromrepo : None
@@ -365,15 +475,22 @@ def list_repo_pkgs(*args, **kwargs):
 
     ret = {}
     for repo in repos:
-        repoquery_cmd = '--all --repoid="{0}"'.format(repo)
+        repoquery_cmd = '--all --repoid="{0}" --show-duplicates'.format(repo)
         for arg in args:
             repoquery_cmd += ' "{0}"'.format(arg)
         all_pkgs = _repoquery_pkginfo(repoquery_cmd)
         for pkg in all_pkgs:
-            ret.setdefault(pkg.repoid, []).append({pkg.name: pkg.version})
+            repo_dict = ret.setdefault(pkg.repoid, {})
+            version_list = repo_dict.setdefault(pkg.name, [])
+            version_list.append(pkg.version)
 
     for reponame in ret:
-        ret[reponame].sort()
+        for pkgname in ret[reponame]:
+            sorted_versions = sorted(
+                [_LooseVersion(x) for x in ret[reponame][pkgname]],
+                reverse=True
+            )
+            ret[reponame][pkgname] = [x.vstring for x in sorted_versions]
     return ret
 
 
@@ -381,6 +498,13 @@ def list_upgrades(refresh=True, **kwargs):
     '''
     Check whether or not an upgrade is available for all packages
 
+    The ``fromrepo``, ``enablerepo``, and ``disablerepo`` arguments are
+    supported, as used in pkg states, and the ``disableexcludes`` option is
+    also supported.
+
+    .. versionadded:: 2014.7.0
+        Support for the ``disableexcludes`` option
+
     CLI Example:
 
     .. code-block:: bash
@@ -388,11 +512,12 @@ def list_upgrades(refresh=True, **kwargs):
         salt '*' pkg.list_upgrades
     '''
     if salt.utils.is_true(refresh):
-        refresh_db()
+        refresh_db(**kwargs)
 
     repo_arg = _get_repo_options(**kwargs)
+    exclude_arg = _get_excludes_option(**kwargs)
     updates = _repoquery_pkginfo(
-        '{0} --all --pkgnarrow=updates'.format(repo_arg)
+        '{0} {1} --all --pkgnarrow=updates'.format(repo_arg, exclude_arg)
     )
     return dict([(x.name, x.version) for x in updates])
 
@@ -409,8 +534,12 @@ def check_db(*names, **kwargs):
     2. If ``found`` is ``False``, then a second key called ``suggestions`` will
        be present, which will contain a list of possible matches.
 
-    The ``fromrepo``, ``enablerepo``, and ``disablerepo`` arguments are
-    supported, as used in pkg states.
+    The ``fromrepo``, ``enablerepo`` and ``disablerepo`` arguments are
+    supported, as used in pkg states, and the ``disableexcludes`` option is
+    also supported.
+
+    .. versionadded:: 2014.7.0
+        Support for the ``disableexcludes`` option
 
     CLI Examples:
 
@@ -418,10 +547,13 @@ def check_db(*names, **kwargs):
 
         salt '*' pkg.check_db <package1> <package2> <package3>
         salt '*' pkg.check_db <package1> <package2> <package3> fromrepo=epel-testing
+        salt '*' pkg.check_db <package1> <package2> <package3> disableexcludes=main
     '''
+    normalize = kwargs.pop('normalize') if kwargs.get('normalize') else False
     repo_arg = _get_repo_options(**kwargs)
+    exclude_arg = _get_excludes_option(**kwargs)
     repoquery_base = \
-        '{0} --all --quiet --whatprovides'.format(repo_arg)
+        '{0} {1} --all --quiet --whatprovides'.format(repo_arg, exclude_arg)
 
     if 'pkg._avail' in __context__:
         avail = __context__['pkg._avail']
@@ -437,7 +569,10 @@ def check_db(*names, **kwargs):
                 name, arch = line.split('_|-')
             except ValueError:
                 continue
-            avail.append(normalize_name('.'.join((name, arch))))
+            if normalize:
+                avail.append(normalize_name('.'.join((name, arch))))
+            else:
+                avail.append('.'.join((name, arch)))
         __context__['pkg._avail'] = avail
 
     ret = {}
@@ -456,14 +591,14 @@ def check_db(*names, **kwargs):
     return ret
 
 
-def refresh_db():
+def refresh_db(**kwargs):
     '''
     Check the yum repos for updated packages
 
     Returns:
 
     - ``True``: Updates are available
-    - ``False``: An error occured
+    - ``False``: An error occurred
     - ``None``: No updates are available
 
     CLI Example:
@@ -477,17 +612,16 @@ def refresh_db():
         0: None,
         1: False,
     }
+    branch_arg = _get_branch_option(**kwargs)
 
-    cmd = 'yum -q clean expire-cache && yum -q check-update'
-    ret = __salt__['cmd.retcode'](cmd,
-                                  output_loglevel='debug',
-                                  ignore_retcode=True)
+    cmd = 'yum -q clean expire-cache && yum -q check-update {0}'.format(branch_arg)
+    ret = __salt__['cmd.retcode'](cmd, ignore_retcode=True)
     return retcodes.get(ret, False)
 
 
-def clean_metadata():
+def clean_metadata(**kwargs):
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.1.0
 
     Cleans local yum metadata. Functionally identical to :mod:`refresh_db()
     <salt.modules.yumpkg.refresh_db>`.
@@ -498,7 +632,7 @@ def clean_metadata():
 
         salt '*' pkg.clean_metadata
     '''
-    return refresh_db()
+    return refresh_db(**kwargs)
 
 
 def group_install(name,
@@ -506,7 +640,7 @@ def group_install(name,
                   include=(),
                   **kwargs):
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.1.0
 
     Install the passed package group(s). This is basically a wrapper around
     pkg.install, which performs package group resolution for the user. This
@@ -601,6 +735,8 @@ def install(name=None,
             skip_verify=False,
             pkgs=None,
             sources=None,
+            reinstall=False,
+            normalize=True,
             **kwargs):
     '''
     Install the passed package(s), add refresh=True to clean the yum database
@@ -626,6 +762,19 @@ def install(name=None,
     refresh
         Whether or not to update the yum database before executing.
 
+    reinstall
+        Specifying reinstall=True will use ``yum reinstall`` rather than
+        ``yum install`` for requested packages that are already installed.
+
+        If a version is specified with the requested package, then
+        ``yum reinstall`` will only be used if the installed version
+        matches the requested version.
+
+        Works with sources when the package header of the source can be
+        matched to the name and version of an installed package.
+
+        .. versionadded:: 2014.7.0
+
     skip_verify
         Skip the GPG verification check (e.g., ``--nogpgcheck``)
 
@@ -648,6 +797,12 @@ def install(name=None,
         Specify an enabled package repository (or repositories) to disable.
         (e.g., ``yum --disablerepo='somerepo'``)
 
+    disableexcludes
+        Disable exclude from main, for a repo or for everything.
+        (e.g., ``yum --disableexcludes='main'``)
+
+        .. versionadded:: 2014.7.0
+
 
     Multiple Package Installation Options:
 
@@ -675,6 +830,20 @@ def install(name=None,
 
             salt '*' pkg.install sources='[{"foo": "salt://foo.rpm"}, {"bar": "salt://bar.rpm"}]'
 
+    normalize
+        Normalize the package name by removing the architecture.  Default is True.
+        This is useful for poorly created packages which might include the
+        architecture as an actual part of the name such as kernel modules
+        which match a specific kernel version.
+
+        .. versionadded:: 2014.7.0
+
+    Example:
+
+    .. code-block:: bash
+
+        salt -G role:nsd pkg.install gpfs.gplbin-2.6.32-279.31.1.el6.x86_64 normalize=False
+
 
     Returns a dict containing the new package names and versions::
 
@@ -682,11 +851,12 @@ def install(name=None,
                        'new': '<new-version>'}}
     '''
     if salt.utils.is_true(refresh):
-        refresh_db()
+        refresh_db(**kwargs)
+    reinstall = salt.utils.is_true(reinstall)
 
     try:
         pkg_params, pkg_type = __salt__['pkg_resource.parse_targets'](
-            name, pkgs, sources, **kwargs
+            name, pkgs, sources, normalize=normalize, **kwargs
         )
     except MinionError as exc:
         raise CommandExecutionError(exc)
@@ -704,64 +874,112 @@ def install(name=None,
                         'package targets')
 
     repo_arg = _get_repo_options(fromrepo=fromrepo, **kwargs)
+    exclude_arg = _get_excludes_option(**kwargs)
+    branch_arg = _get_branch_option(**kwargs)
 
     old = list_pkgs()
+    targets = []
     downgrade = []
+    to_reinstall = {}
     if pkg_type == 'repository':
-        targets = []
-        for pkgname, version_num in pkg_params.iteritems():
-            if version_num is None:
-                targets.append(pkgname)
-            else:
-                cver = old.get(pkgname, '')
-                arch = ''
-                try:
-                    namepart, archpart = pkgname.rsplit('.', 1)
-                except ValueError:
-                    pass
-                else:
-                    if archpart in __ARCHES:
-                        arch = '.' + archpart
-                        pkgname = namepart
-
-                pkgstr = '"{0}-{1}{2}"'.format(pkgname, version_num, arch)
-                if not cver or salt.utils.compare_versions(ver1=version_num,
-                                                           oper='>=',
-                                                           ver2=cver):
-                    targets.append(pkgstr)
-                else:
-                    downgrade.append(pkgstr)
+        pkg_params_items = pkg_params.iteritems()
     else:
-        targets = pkg_params
+        pkg_params_items = []
+        for pkg_source in pkg_params:
+            rpm_info = _rpm_pkginfo(pkg_source)
+            if rpm_info is not None:
+                pkg_params_items.append([rpm_info.name, rpm_info.version, pkg_source])
+            else:
+                pkg_params_items.append([pkg_source, None, pkg_source])
+
+    for pkg_item_list in pkg_params_items:
+        pkgname = pkg_item_list[0]
+        version_num = pkg_item_list[1]
+        if version_num is None:
+            if reinstall and pkg_type == 'repository' and pkgname in old:
+                to_reinstall[pkgname] = pkgname
+            else:
+                targets.append(pkgname)
+        else:
+            cver = old.get(pkgname, '')
+            arch = ''
+            try:
+                namepart, archpart = pkgname.rsplit('.', 1)
+            except ValueError:
+                pass
+            else:
+                if archpart in __ARCHES:
+                    arch = '.' + archpart
+                    pkgname = namepart
+
+            if pkg_type == 'repository':
+                pkgstr = '"{0}-{1}{2}"'.format(pkgname, version_num, arch)
+            else:
+                pkgstr = pkg_item_list[2]
+            if reinstall and cver \
+                    and salt.utils.compare_versions(ver1=version_num,
+                                                    oper='==',
+                                                    ver2=cver):
+                to_reinstall[pkgname] = pkgstr
+            elif not cver or salt.utils.compare_versions(ver1=version_num,
+                                                         oper='>=',
+                                                         ver2=cver):
+                targets.append(pkgstr)
+            else:
+                downgrade.append(pkgstr)
 
     if targets:
-        cmd = 'yum -y {repo} {gpgcheck} install {pkg}'.format(
+        cmd = 'yum -y {repo} {exclude} {branch} {gpgcheck} install {pkg}'.format(
             repo=repo_arg,
+            exclude=exclude_arg,
+            branch=branch_arg,
             gpgcheck='--nogpgcheck' if skip_verify else '',
             pkg=' '.join(targets),
         )
-        __salt__['cmd.run'](cmd, output_loglevel='debug')
+        __salt__['cmd.run'](cmd, output_loglevel='trace')
 
     if downgrade:
-        cmd = 'yum -y {repo} {gpgcheck} downgrade {pkg}'.format(
+        cmd = 'yum -y {repo} {exclude} {branch} {gpgcheck} downgrade {pkg}'.format(
             repo=repo_arg,
+            exclude=exclude_arg,
+            branch=branch_arg,
             gpgcheck='--nogpgcheck' if skip_verify else '',
             pkg=' '.join(downgrade),
         )
-        __salt__['cmd.run'](cmd, output_loglevel='debug')
+        __salt__['cmd.run'](cmd, output_loglevel='trace')
+
+    if to_reinstall:
+        cmd = 'yum -y {repo} {exclude} {branch} {gpgcheck} reinstall {pkg}'.format(
+            repo=repo_arg,
+            exclude=exclude_arg,
+            branch=branch_arg,
+            gpgcheck='--nogpgcheck' if skip_verify else '',
+            pkg=' '.join(to_reinstall.values()),
+        )
+        __salt__['cmd.run'](cmd, output_loglevel='trace')
 
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
     ret = salt.utils.compare_dicts(old, new)
+    for pkgname in to_reinstall:
+        if not pkgname not in old:
+            ret.update({pkgname: {'old': old.get(pkgname, ''),
+                                  'new': new.get(pkgname, '')}})
+        else:
+            if pkgname not in ret:
+                ret.update({pkgname: {'old': old.get(pkgname, ''),
+                                      'new': new.get(pkgname, '')}})
     if ret:
         __context__.pop('pkg._avail', None)
     return ret
 
 
-def upgrade(refresh=True):
+def upgrade(refresh=True, fromrepo=None, skip_verify=False, **kwargs):
     '''
     Run a full system upgrade, a yum upgrade
 
+    .. versionchanged:: 2014.7.0
+
     Return a dict containing the new package names and versions::
 
         {'<package>': {'old': '<old-version>',
@@ -772,12 +990,42 @@ def upgrade(refresh=True):
     .. code-block:: bash
 
         salt '*' pkg.upgrade
+
+    Repository Options:
+
+    fromrepo
+        Specify a package repository (or repositories) from which to install.
+        (e.g., ``yum --disablerepo='*' --enablerepo='somerepo'``)
+
+    enablerepo (ignored if ``fromrepo`` is specified)
+        Specify a disabled package repository (or repositories) to enable.
+        (e.g., ``yum --enablerepo='somerepo'``)
+
+    disablerepo (ignored if ``fromrepo`` is specified)
+        Specify an enabled package repository (or repositories) to disable.
+        (e.g., ``yum --disablerepo='somerepo'``)
+
+    disableexcludes
+        Disable exclude from main, for a repo or for everything.
+        (e.g., ``yum --disableexcludes='main'``)
+
+        .. versionadded:: 2014.7.0
     '''
     if salt.utils.is_true(refresh):
-        refresh_db()
+        refresh_db(**kwargs)
+
+    repo_arg = _get_repo_options(fromrepo=fromrepo, **kwargs)
+    exclude_arg = _get_excludes_option(**kwargs)
+    branch_arg = _get_branch_option(**kwargs)
+
     old = list_pkgs()
-    cmd = 'yum -q -y upgrade'
-    __salt__['cmd.run'](cmd, output_loglevel='debug')
+    cmd = 'yum -q -y {repo} {exclude} {branch} {gpgcheck} upgrade'.format(
+        repo=repo_arg,
+        exclude=exclude_arg,
+        branch=branch_arg,
+        gpgcheck='--nogpgcheck' if skip_verify else '')
+
+    __salt__['cmd.run'](cmd, output_loglevel='trace')
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
     ret = salt.utils.compare_dicts(old, new)
@@ -823,7 +1071,7 @@ def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=W0613
     if not targets:
         return {}
     cmd = 'yum -q -y remove "{0}"'.format('" "'.join(targets))
-    __salt__['cmd.run'](cmd, output_loglevel='debug')
+    __salt__['cmd.run'](cmd, output_loglevel='trace')
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
     ret = salt.utils.compare_dicts(old, new)
@@ -863,24 +1111,249 @@ def purge(name=None, pkgs=None, **kwargs):  # pylint: disable=W0613
     return remove(name=name, pkgs=pkgs)
 
 
-def verify(*names):
+def hold(name=None, pkgs=None, sources=None, **kwargs):  # pylint: disable=W0613
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.7.0
+
+    Hold packages with ``yum -q versionlock``.
+
+    name
+        The name of the package to be deleted.
+
+    Multiple Package Options:
+
+    pkgs
+        A list of packages to hold. Must be passed as a python list. The
+        ``name`` parameter will be ignored if this option is passed.
+
+    Returns a dict containing the changes.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.hold <package name>
+        salt '*' pkg.hold pkgs='["foo", "bar"]'
+    '''
+    if 'yum-plugin-versionlock' not in list_pkgs():
+        raise SaltInvocationError(
+            'Packages cannot be held, yum-plugin-versionlock is not installed.'
+        )
+    if not name and not pkgs and not sources:
+        raise SaltInvocationError(
+            'One of name, pkgs, or sources must be specified.'
+        )
+    if pkgs and sources:
+        raise SaltInvocationError(
+            'Only one of pkgs or sources can be specified.'
+        )
+
+    targets = []
+    if pkgs:
+        for pkg in pkgs:
+            ret = check_db(pkg)
+            if not ret[pkg]['found']:
+                raise SaltInvocationError(
+                    'Package {0} not available in repository.'.format(name)
+                )
+        targets.extend(pkgs)
+    elif sources:
+        for source in sources:
+            targets.append(next(iter(source)))
+    else:
+        ret = check_db(name)
+        if not ret[name]['found']:
+            raise SaltInvocationError(
+                'Package {0} not available in repository.'.format(name)
+            )
+        targets.append(name)
+
+    current_locks = get_locked_packages(full=False)
+    ret = {}
+    for target in targets:
+        if isinstance(target, dict):
+            target = next(iter(target))
+
+        ret[target] = {'name': target,
+                       'changes': {},
+                       'result': False,
+                       'comment': ''}
+
+        if target not in current_locks:
+            if 'test' in __opts__ and __opts__['test']:
+                ret[target].update(result=None)
+                ret[target]['comment'] = ('Package {0} is set to be held.'
+                                          .format(target))
+            else:
+                cmd = 'yum -q versionlock {0}'.format(target)
+                out = __salt__['cmd.run_all'](cmd)
+
+                if out['retcode'] == 0:
+                    ret[target].update(result=True)
+                    ret[target]['comment'] = ('Package {0} is now being held.'
+                                              .format(target))
+                    ret[target]['changes']['new'] = 'hold'
+                    ret[target]['changes']['old'] = ''
+                else:
+                    ret[target]['comment'] = ('Package {0} was unable to be held.'
+                                              .format(target))
+        else:
+            ret[target].update(result=True)
+            ret[target]['comment'] = ('Package {0} is already set to be held.'
+                                      .format(target))
+    return ret
+
+
+def unhold(name=None, pkgs=None, sources=None, **kwargs):  # pylint: disable=W0613
+    '''
+    .. versionadded:: 2014.7.0
+
+    Hold packages with ``yum -q versionlock``.
+
+    name
+        The name of the package to be deleted.
+
+    Multiple Package Options:
+
+    pkgs
+        A list of packages to unhold. Must be passed as a python list. The
+        ``name`` parameter will be ignored if this option is passed.
+
+    Returns a dict containing the changes.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.unhold <package name>
+        salt '*' pkg.unhold pkgs='["foo", "bar"]'
+    '''
+    if 'yum-plugin-versionlock' not in list_pkgs():
+        raise SaltInvocationError(
+            'Packages cannot be unheld, yum-plugin-versionlock is not installed.'
+        )
+    if not name and not pkgs and not sources:
+        raise SaltInvocationError(
+            'One of name, pkgs, or sources must be specified.'
+        )
+    if pkgs and sources:
+        raise SaltInvocationError(
+            'Only one of pkgs or sources can be specified.'
+        )
+
+    targets = []
+    if pkgs:
+        targets.extend(pkgs)
+    elif sources:
+        for source in sources:
+            targets.append(next(iter(source)))
+    else:
+        targets.append(name)
+
+    current_locks = get_locked_packages(full=True)
+    ret = {}
+    for target in targets:
+        if isinstance(target, dict):
+            target = next(iter(target))
+
+        ret[target] = {'name': target,
+                       'changes': {},
+                       'result': False,
+                       'comment': ''}
+
+        search_locks = [lock for lock in current_locks
+                        if target in lock]
+        if search_locks:
+            if 'test' in __opts__ and __opts__['test']:
+                ret[target].update(result=None)
+                ret[target]['comment'] = ('Package {0} is set to be unheld.'
+                                          .format(target))
+            else:
+                _targets = ' '.join('"' + item + '"' for item in search_locks)
+                cmd = 'yum -q versionlock delete {0}'.format(_targets)
+                out = __salt__['cmd.run_all'](cmd)
+
+                if out['retcode'] == 0:
+                    ret[target].update(result=True)
+                    ret[target]['comment'] = ('Package {0} is no longer held.'
+                                              .format(target))
+                    ret[target]['changes']['new'] = ''
+                    ret[target]['changes']['old'] = 'hold'
+                else:
+                    ret[target]['comment'] = ('Package {0} was unable to be '
+                                              'unheld.'.format(target))
+        else:
+            ret[target].update(result=True)
+            ret[target]['comment'] = ('Package {0} is not being held.'
+                                      .format(target))
+    return ret
+
+
+def get_locked_packages(pattern=None, full=True):
+    '''
+    Get packages that are currently locked
+    ``yum -q versionlock list``.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.get_locked_packages
+    '''
+    cmd = 'yum -q versionlock list'
+    ret = __salt__['cmd.run'](cmd).split('\n')
+
+    if pattern:
+        if full:
+            _pat = r'(\d\:{0}\-\S+)'.format(pattern)
+        else:
+            _pat = r'\d\:({0}\-\S+)'.format(pattern)
+    else:
+        if full:
+            _pat = r'(\d\:\w+\-\S+)'
+        else:
+            _pat = r'\d\:(\w+\-\S+)'
+    pat = re.compile(_pat)
+
+    current_locks = []
+    for item in ret:
+        match = pat.search(item)
+        if match:
+            if not full:
+                woarch = match.group(1).rsplit('.', 1)[0]
+                worel = woarch.rsplit('-', 1)[0]
+                wover = worel.rsplit('-', 1)[0]
+                _match = wover
+            else:
+                _match = match.group(1)
+            current_locks.append(_match)
+    return current_locks
+
+
+def verify(*names, **kwargs):
+    '''
+    .. versionadded:: 2014.1.0
 
     Runs an rpm -Va on a system, and returns the results in a dict
 
+    Files with an attribute of config, doc, ghost, license or readme in the
+    package header can be ignored using the ``ignore_types`` keyword argument
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' pkg.verify
+        salt '*' pkg.verify httpd
+        salt '*' pkg.verify 'httpd postfix'
+        salt '*' pkg.verify 'httpd postfix' ignore_types=['config','doc']
     '''
-    return __salt__['lowpkg.verify'](*names)
+    return __salt__['lowpkg.verify'](*names, **kwargs)
 
 
 def group_list():
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.1.0
 
     Lists all groups known by yum on this system
 
@@ -892,7 +1365,7 @@ def group_list():
     '''
     ret = {'installed': [], 'available': [], 'available languages': {}}
     cmd = 'yum grouplist'
-    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='debug').splitlines()
+    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace').splitlines()
     key = None
     for idx in xrange(len(out)):
         if out[idx] == 'Installed Groups:':
@@ -925,7 +1398,7 @@ def group_list():
 
 def group_info(name):
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.1.0
 
     Lists packages belonging to a certain group
 
@@ -946,7 +1419,7 @@ def group_info(name):
     cmd_template = 'repoquery --plugins --group --grouppkgs={0} --list {1!r}'
 
     cmd = cmd_template.format('all', name)
-    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='debug')
+    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
     all_pkgs = set(out.splitlines())
 
     if not all_pkgs:
@@ -956,7 +1429,7 @@ def group_info(name):
         cmd = cmd_template.format(pkgtype, name)
         packages = set(
             __salt__['cmd.run_stdout'](
-                cmd, output_loglevel='debug'
+                cmd, output_loglevel='trace'
             ).splitlines()
         )
         ret['{0} packages'.format(pkgtype)].extend(sorted(packages))
@@ -968,7 +1441,7 @@ def group_info(name):
     ret['conditional packages'] = sorted(all_pkgs)
 
     cmd = 'repoquery --plugins --group --info {0!r}'.format(name)
-    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='debug')
+    out = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
     if out:
         ret['description'] = '\n'.join(out.splitlines()[1:]).strip()
 
@@ -977,7 +1450,7 @@ def group_info(name):
 
 def group_diff(name):
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.1.0
 
     Lists packages belonging to a certain group, and which are installed
 
@@ -1030,7 +1503,7 @@ def list_repos(basedir='/etc/yum.repos.d'):
 
 def get_repo(repo, basedir='/etc/yum.repos.d', **kwargs):  # pylint: disable=W0613
     '''
-    Display a repo from <basedir> (default basedir: /etc/yum.repos.d).
+    Display a repo from <basedir> (default basedir: ``/etc/yum.repos.d``).
 
     CLI Examples:
 
@@ -1046,12 +1519,12 @@ def get_repo(repo, basedir='/etc/yum.repos.d', **kwargs):  # pylint: disable=W06
     for arepo in repos.keys():
         if arepo == repo:
             repofile = repos[arepo]['file']
-    if not repofile:
-        raise Exception('repo {0} was not found in {1}'.format(repo, basedir))
 
-    # Return just one repo
-    filerepos = _parse_repo_file(repofile)[1]
-    return filerepos[repo]
+    if repofile:
+        # Return just one repo
+        filerepos = _parse_repo_file(repofile)[1]
+        return filerepos[repo]
+    return {}
 
 
 def del_repo(repo, basedir='/etc/yum.repos.d', **kwargs):  # pylint: disable=W0613
@@ -1269,18 +1742,23 @@ def _parse_repo_file(filename):
 
             # These are the actual configuration lines that matter
             if '=' in line:
-                comps = line.strip().split('=')
-                repos[repo][comps[0].strip()] = '='.join(comps[1:])
+                try:
+                    comps = line.strip().split('=')
+                    repos[repo][comps[0].strip()] = '='.join(comps[1:])
+                except KeyError:
+                    log.error('Failed to parse line in {0}, '
+                              'offending line was "{1}"'.format(filename,
+                                                                line.rstrip()))
 
     return (header, repos)
 
 
 def file_list(*packages):
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.1.0
 
     List the files that belong to a package. Not specifying any packages will
-    return a list of _every_ file on the system's rpm database (not generally
+    return a list of *every* file on the system's rpm database (not generally
     recommended).
 
     CLI Examples:
@@ -1296,10 +1774,10 @@ def file_list(*packages):
 
 def file_dict(*packages):
     '''
-    .. versionadded:: 2014.1.0 (Hydrogen)
+    .. versionadded:: 2014.1.0
 
     List the files that belong to a package, grouped by package. Not
-    specifying any packages will return a list of _every_ file on the system's
+    specifying any packages will return a list of *every* file on the system's
     rpm database (not generally recommended).
 
     CLI Examples:
@@ -1323,3 +1801,34 @@ def expand_repo_def(repokwargs):
     '''
     # YUM doesn't need the data massaged.
     return repokwargs
+
+
+def owner(*paths):
+    '''
+    .. versionadded:: 2014.7.0
+
+    Return the name of the package that owns the file. Multiple file paths can
+    be passed. Like :mod:`pkg.version <salt.modules.yumpkg.version`, if a
+    single path is passed, a string will be returned, and if multiple paths are
+    passed, a dictionary of file/package name pairs will be returned.
+
+    If the file is not owned by a package, or is not present on the minion,
+    then an empty string will be returned for that path.
+
+    CLI Example:
+
+        salt '*' pkg.owner /usr/bin/apachectl
+        salt '*' pkg.owner /usr/bin/apachectl /etc/httpd/conf/httpd.conf
+    '''
+    if not paths:
+        return ''
+    ret = {}
+    cmd = 'rpm -qf --queryformat "%{{NAME}}" {0!r}'
+    for path in paths:
+        ret[path] = __salt__['cmd.run_stdout'](cmd.format(path),
+                                               output_loglevel='trace')
+        if 'not owned' in ret[path].lower():
+            ret[path] = ''
+    if len(ret) == 1:
+        return ret.values()[0]
+    return ret
