@@ -7,13 +7,24 @@ Package support for openSUSE via the zypper package manager
 import copy
 import logging
 import re
+from contextlib import contextmanager as _contextmanager
 
 # Import salt libs
 import salt.utils
-from salt.exceptions import CommandExecutionError, MinionError
+from salt.utils.decorators import depends as _depends
+from salt.exceptions import (
+    CommandExecutionError, MinionError, SaltInvocationError)
 
 log = logging.getLogger(__name__)
 
+HAS_ZYPP = False
+
+try:
+    import zypp
+    HAS_ZYPP = True
+except ImportError:
+    pass
+
 # Define the module's virtual name
 __virtualname__ = 'pkg'
 
@@ -22,6 +33,8 @@ def __virtual__():
     '''
     Set the virtual pkg module if the os is openSUSE
     '''
+    if not HAS_ZYPP:
+        return False
     if __grains__.get('os_family', '') != 'Suse':
         return False
     # Not all versions of Suse use zypper, check that it is available
@@ -44,7 +57,7 @@ def list_upgrades(refresh=True):
         refresh_db()
     ret = {}
     out = __salt__['cmd.run_stdout'](
-        'zypper list-updates', output_loglevel='debug'
+        'zypper list-updates', output_loglevel='trace'
     )
     for line in out.splitlines():
         if not line:
@@ -98,7 +111,7 @@ def latest_version(*names, **kwargs):
     # Split call to zypper into batches of 500 packages
     while restpackages:
         cmd = 'zypper info -t package {0}'.format(' '.join(restpackages[:500]))
-        output = __salt__['cmd.run_stdout'](cmd, output_loglevel='debug')
+        output = __salt__['cmd.run_stdout'](cmd, output_loglevel='trace')
         outputs.extend(re.split('Information for package \\S+:\n', output))
         restpackages = restpackages[500:]
     for package in outputs:
@@ -187,7 +200,7 @@ def list_pkgs(versions_as_list=False, **kwargs):
 
     cmd = 'rpm -qa --queryformat "%{NAME}_|-%{VERSION}_|-%{RELEASE}\\n"'
     ret = {}
-    out = __salt__['cmd.run'](cmd, output_loglevel='debug')
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
     for line in out.splitlines():
         name, pkgver, rel = line.split('_|-')
         if rel:
@@ -201,6 +214,305 @@ def list_pkgs(versions_as_list=False, **kwargs):
     return ret
 
 
+class _RepoInfo(object):
+    '''
+    Incapsulate all properties that are dumped in zypp._RepoInfo.dumpOn:
+    http://doc.opensuse.org/projects/libzypp/HEAD/classzypp_1_1RepoInfo.html#a2ba8fdefd586731621435428f0ec6ff1
+    '''
+    repo_types = {}
+
+    if HAS_ZYPP:
+        repo_types = {
+            zypp.RepoType.NONE_e: 'NONE',
+            zypp.RepoType.RPMMD_e: 'rpm-md',
+            zypp.RepoType.YAST2_e: 'yast2',
+            zypp.RepoType.RPMPLAINDIR_e: 'plaindir',
+        }
+
+    def __init__(self, zypp_repo_info=None):
+        self.zypp = zypp_repo_info if zypp_repo_info else zypp.RepoInfo()
+
+    @property
+    def options(self):
+        class_items = self.__class__.__dict__.iteritems()
+        return dict([(k, getattr(self, k)) for k, v in class_items
+                     if isinstance(v, property) and k != 'options'
+                     and getattr(self, k) not in (None, '')])
+
+    def _check_only_mirrorlist_or_url(self):
+        if all(x in self.options for x in ('mirrorlist', 'url')):
+            raise ValueError(
+                'Only one of \'mirrorlist\' and \'url\' can be specified')
+
+    def _zypp_url(self, url):
+        return zypp.Url(url) if url else zypp.Url()
+
+    @options.setter
+    def options(self, value):
+        for k, v in value.iteritems():
+            setattr(self, k, v)
+
+    @property
+    def alias(self):
+        return self.zypp.alias()
+
+    @alias.setter
+    def alias(self, value):
+        if value:
+            self.zypp.setAlias(value)
+        else:
+            raise ValueError('Alias cannot be empty')
+
+    @property
+    def autorefresh(self):
+        return self.zypp.autorefresh()
+
+    @autorefresh.setter
+    def autorefresh(self, value):
+        self.zypp.setAutorefresh(value)
+
+    @property
+    def enabled(self):
+        return self.zypp.enabled()
+
+    @enabled.setter
+    def enabled(self, value):
+        self.zypp.setEnabled(value)
+
+    @property
+    def gpgcheck(self):
+        return self.zypp.gpgCheck()
+
+    @gpgcheck.setter
+    def gpgcheck(self, value):
+        self.zypp.setGpgCheck(value)
+
+    @property
+    def gpgkey(self):
+        return self.zypp.gpgKeyUrl().asCompleteString()
+
+    @gpgkey.setter
+    def gpgkey(self, value):
+        self.zypp.setGpgKeyUrl(self._zypp_url(value))
+
+    @property
+    def keeppackages(self):
+        return self.zypp.keepPackages()
+
+    @keeppackages.setter
+    def keeppackages(self, value):
+        self.zypp.setKeepPackages(value)
+
+    @property
+    def metadataPath(self):
+        return self.zypp.metadataPath().c_str()
+
+    @metadataPath.setter
+    def metadataPath(self, value):
+        self.zypp.setMetadataPath(value)
+
+    @property
+    def mirrorlist(self):
+        return self.zypp.mirrorListUrl().asCompleteString()
+
+    @mirrorlist.setter
+    def mirrorlist(self, value):
+        self.zypp.setMirrorListUrl(self._zypp_url(value))
+        # self._check_only_mirrorlist_or_url()
+
+    @property
+    def name(self):
+        return self.zypp.name()
+
+    @name.setter
+    def name(self, value):
+        self.zypp.setName(value)
+
+    @property
+    def packagesPath(self):
+        return self.zypp.packagesPath().c_str()
+
+    @packagesPath.setter
+    def packagesPath(self, value):
+        self.zypp.setPackagesPath(self._zypp_url(value))
+
+    @property
+    def path(self):
+        return self.zypp.path().c_str()
+
+    @path.setter
+    def path(self, value):
+        self.zypp.setPath(self._zypp_url(value))
+
+    @property
+    def priority(self):
+        return self.zypp.priority()
+
+    @priority.setter
+    def priority(self, value):
+        self.zypp.setPriority(value)
+
+    @property
+    def service(self):
+        return self.zypp.service()
+
+    @service.setter
+    def service(self, value):
+        self.zypp.setService(value)
+
+    @property
+    def targetdistro(self):
+        return self.zypp.targetDistribution()
+
+    @targetdistro.setter
+    def targetdistro(self, value):
+        self.zypp.setTargetDistribution(value)
+
+    @property
+    def type(self):
+        return self.repo_types[self.zypp.type().toEnum()]
+
+    @type.setter
+    def type(self, value):
+        self.zypp.setType(next(k for k, v in self.repo_types if v == value))
+
+    @property
+    def url(self):
+        return self.zypp.url().asCompleteString()
+
+    @url.setter
+    def url(self, value):
+        self.zypp.setBaseUrl(self._zypp_url(value))
+        # self._check_only_mirrorlist_or_url()
+
+
+@_contextmanager
+def _try_zypp():
+    '''
+    Convert errors like:
+    'RuntimeError: [|] Repository has no alias defined.'
+    into
+    'ERROR: Repository has no alias defined.'.
+    '''
+    try:
+        yield
+    except RuntimeError as e:
+        raise CommandExecutionError(re.sub(r'\[.*\] ', '', str(e)))
+
+
+@_depends('zypp')
+def _get_zypp_repo(repo, **kwargs):
+    '''
+    Get zypp._RepoInfo object by repo alias.
+    '''
+    with _try_zypp():
+        return zypp.RepoManager().getRepositoryInfo(repo)
+
+
+@_depends('zypp')
+def get_repo(repo, **kwargs):
+    '''
+    Display a repo.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' pkg.get_repo alias
+    '''
+    try:
+        r = _RepoInfo(_get_zypp_repo(repo))
+    except CommandExecutionError:
+        return {}
+    return r.options
+
+
+@_depends('zypp')
+def list_repos():
+    '''
+    Lists all repos.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+       salt '*' pkg.list_repos
+    '''
+    with _try_zypp():
+        ret = {}
+        for r in zypp.RepoManager().knownRepositories():
+            ret[r.alias()] = get_repo(r.alias())
+        return ret
+
+
+@_depends('zypp')
+def del_repo(repo, **kwargs):
+    '''
+    Delete a repo.
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' pkg.del_repo alias
+        salt '*' pkg.del_repo alias
+    '''
+    r = _get_zypp_repo(repo)
+    with _try_zypp():
+        zypp.RepoManager().removeRepository(r)
+    return 'File {1} containing repo {0!r} has been removed.\n'.format(
+        repo, r.path().c_str())
+
+
+@_depends('zypp')
+def mod_repo(repo, **kwargs):
+    '''
+    Modify one or more values for a repo. If the repo does not exist, it will
+    be created, so long as the following values are specified:
+
+    repo
+        alias by which the zypper refers to the repo
+    url or mirrorlist
+        the URL for zypper to reference
+
+    Key/Value pairs may also be removed from a repo's configuration by setting
+    a key to a blank value. Bear in mind that a name cannot be deleted, and a
+    url can only be deleted if a mirrorlist is specified (or vice versa).
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' pkg.mod_repo alias alias=new_alias
+        salt '*' pkg.mod_repo alias enabled=True
+        salt '*' pkg.mod_repo alias url= mirrorlist=http://host.com/
+    '''
+    # Filter out '__pub' arguments, as well as saltenv
+    repo_opts = {}
+    for x in kwargs:
+        if not x.startswith('__') and x not in ('saltenv',):
+            repo_opts[x] = kwargs[x]
+
+    repo_manager = zypp.RepoManager()
+    try:
+        r = _RepoInfo(repo_manager.getRepositoryInfo(repo))
+        new_repo = False
+    except RuntimeError:
+        r = _RepoInfo()
+        r.alias = repo
+        new_repo = True
+    try:
+        r.options = repo_opts
+    except ValueError as e:
+        raise SaltInvocationError(str(e))
+    with _try_zypp():
+        if new_repo:
+            repo_manager.addRepository(r.zypp)
+        else:
+            repo_manager.modifyRepository(repo, r.zypp)
+    return r.options
+
+
 def refresh_db():
     '''
     Just run a ``zypper refresh``, return a dict::
@@ -215,7 +527,7 @@ def refresh_db():
     '''
     cmd = 'zypper refresh'
     ret = {}
-    out = __salt__['cmd.run'](cmd, output_loglevel='debug')
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace')
     for line in out.splitlines():
         if not line:
             continue
@@ -366,7 +678,7 @@ def install(name=None,
             .format(fromrepoopt, '" "'.join(targets[:500]))
         )
         targets = targets[500:]
-        out = __salt__['cmd.run'](cmd, output_loglevel='debug')
+        out = __salt__['cmd.run'](cmd, output_loglevel='trace')
         for line in out.splitlines():
             match = re.match(
                 "^The selected package '([^']+)'.+has lower version",
@@ -381,7 +693,7 @@ def install(name=None,
             '--auto-agree-with-licenses --force {0}{1}'
             .format(fromrepoopt, ' '.join(downgrades[:500]))
         )
-        __salt__['cmd.run'](cmd, output_loglevel='debug')
+        __salt__['cmd.run'](cmd, output_loglevel='trace')
         downgrades = downgrades[500:]
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
@@ -407,7 +719,7 @@ def upgrade(refresh=True):
         refresh_db()
     old = list_pkgs()
     cmd = 'zypper --non-interactive update --auto-agree-with-licenses'
-    __salt__['cmd.run'](cmd, output_loglevel='debug')
+    __salt__['cmd.run'](cmd, output_loglevel='trace')
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
     return salt.utils.compare_dicts(old, new)
@@ -433,7 +745,7 @@ def _uninstall(action='remove', name=None, pkgs=None):
             'zypper --non-interactive remove {0} {1}'
             .format(purge_arg, ' '.join(targets[:500]))
         )
-        __salt__['cmd.run'](cmd, output_loglevel='debug')
+        __salt__['cmd.run'](cmd, output_loglevel='trace')
         targets = targets[500:]
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
