@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 '''
+Outputter for displaying results of state runs
+==============================================
+
 The return data from the Highstate command is a standard data structure
 which is parsed by the highstate outputter to deliver a clean and readable
 set of information about the HighState run on minions.
@@ -13,18 +16,44 @@ state_verbose:
     instruct the highstate outputter to omit displaying anything in green, this
     means that nothing with a result of True and no changes will not be printed
 state_output:
-    The highstate outputter has three output modes, `full`, `terse`, `mixed`,
-    and `changes`. The default is set to full, which will display many lines of
-    detailed information for each executed chunk. If the `state_output` option
-    is set to `terse` then the output is greatly simplified and shown in only
-    one line.  If `mixed` is used, then terse output will be used unless a
+    The highstate outputter has five output modes, `full`, `terse`, `mixed`,
+    `changes` and `filter`. The default is set to full, which will display many
+    lines of detailed information for each executed chunk. If the `state_output`
+    option is set to `terse` then the output is greatly simplified and shown in
+    only one line.  If `mixed` is used, then terse output will be used unless a
     state failed, in which case full output will be used.  If `changes` is used,
     then terse output will be used if there was no error and no changes,
-    otherwise full output will be used.
+    otherwise full output will be used. If `filter` is used, then either or both
+    of two different filters can be used: `exclude` or `terse`. These can be set
+    as such from the command line, or in the Salt config as
+    `state_output_exclude` or `state_output_terse`, respectively. The values to
+    exclude must be a comma-separated list of `True`, `False` and/or `None`.
+    Because of parsing nuances, if only one of these is used, it must still
+    contain a comma. For instance: `exclude=True,`.
 state_tabular:
     If `state_output` uses the terse output, set this to `True` for an aligned
     output format.  If you wish to use a custom format, this can be set to a
     string.
+
+Example output::
+
+    myminion:
+    ----------
+              ID: test.ping
+        Function: module.run
+          Result: True
+         Comment: Module function test.ping executed
+         Changes:
+                  ----------
+                  ret:
+                      True
+
+    Summary
+    ------------
+    Succeeded: 1
+    Failed:    0
+    ------------
+    Total:     0
 '''
 
 # Import python libs
@@ -32,13 +61,14 @@ import pprint
 
 # Import salt libs
 import salt.utils
+import salt.output
+from salt._compat import string_types
 
 
 def output(data):
     '''
-    The HighState Outputter is only meant to
-    be used with the state.highstate function, or a function that returns
-    highstate return data.
+    The HighState Outputter is only meant to be used with the state.highstate
+    function, or a function that returns highstate return data.
     '''
     for host, hostdata in data.iteritems():
         return _format_host(host, hostdata)[0]
@@ -50,14 +80,17 @@ def _format_host(host, data):
     rcounts = {}
     hcolor = colors['GREEN']
     hstrs = []
-    changed = False
+    nchanges = 0
+    strip_colors = __opts__.get('strip_colors', True)
     if isinstance(data, list):
         # Errors have been detected, list them in RED!
         hcolor = colors['RED_BOLD']
-        hstrs.append(('    {0}Data failed to compile:{1[ENDC]}'
+        hstrs.append((u'    {0}Data failed to compile:{1[ENDC]}'
                       .format(hcolor, colors)))
         for err in data:
-            hstrs.append(('{0}----------\n    {1}{2[ENDC]}'
+            if strip_colors:
+                err = salt.output.strip_esc_sequence(err)
+            hstrs.append((u'{0}----------\n    {1}{2[ENDC]}'
                           .format(hcolor, err, colors)))
     if isinstance(data, dict):
         # Strip out the result: True, without changes returns if
@@ -67,7 +100,7 @@ def _format_host(host, data):
         # Verify that the needed data is present
         for tname, info in data.items():
             if '__run_num__' not in info:
-                err = ('The State execution failed to record the order '
+                err = (u'The State execution failed to record the order '
                        'in which all states were executed. The state '
                        'return missing data is:')
                 hstrs.insert(0, pprint.pformat(info))
@@ -80,9 +113,16 @@ def _format_host(host, data):
             # Increment result counts
             rcounts.setdefault(ret['result'], 0)
             rcounts[ret['result']] += 1
+
             tcolor = colors['GREEN']
             schanged, ctext = _format_changes(ret['changes'])
-            changed = changed or schanged
+            nchanges += 1 if schanged else 0
+
+            # Skip this state if it was successful & diff output was requested
+            if __opts__.get('state_output_diff', False) and \
+               ret['result'] and not schanged:
+                continue
+
             if schanged:
                 tcolor = colors['CYAN']
             if ret['result'] is False:
@@ -92,7 +132,40 @@ def _format_host(host, data):
                 hcolor = colors['YELLOW']
                 tcolor = colors['YELLOW']
             comps = tname.split('_|-')
-            if __opts__.get('state_output', 'full').lower() == 'terse':
+            if __opts__.get('state_output', 'full').lower() == 'filter':
+                # By default, full data is shown for all types. However, return
+                # data may be excluded by setting state_output_exclude to a
+                # comma-separated list of True, False or None, or including the
+                # same list with the exclude option on the command line. For
+                # now, this option must include a comma. For example:
+                #     exclude=True,
+                # The same functionality is also available for making return
+                # data terse, instead of excluding it.
+                cliargs = __opts__.get('arg', [])
+                clikwargs = {}
+                for item in cliargs:
+                    if isinstance(item, dict) and '__kwarg__' in item:
+                        clikwargs = item.copy()
+
+                exclude = clikwargs.get(
+                    'exclude', __opts__.get('state_output_exclude', [])
+                )
+                if isinstance(exclude, string_types):
+                    exclude = str(exclude).split(',')
+
+                terse = clikwargs.get(
+                    'terse', __opts__.get('state_output_terse', [])
+                )
+                if isinstance(terse, string_types):
+                    terse = str(terse).split(',')
+
+                if str(ret['result']) in terse:
+                    msg = _format_terse(tcolor, comps, ret, colors, tabular)
+                    hstrs.append(msg)
+                    continue
+                if str(ret['result']) in exclude:
+                    continue
+            elif __opts__.get('state_output', 'full').lower() == 'terse':
                 # Print this chunk in a terse way and continue in the
                 # loop
                 msg = _format_terse(tcolor, comps, ret, colors, tabular)
@@ -112,21 +185,37 @@ def _format_host(host, data):
                     hstrs.append(msg)
                     continue
             state_lines = [
-                '{tcolor}----------{colors[ENDC]}',
-                '    {tcolor}      ID: {comps[1]}{colors[ENDC]}',
-                '    {tcolor}Function: {comps[0]}.{comps[3]}{colors[ENDC]}',
-                '    {tcolor}  Result: {ret[result]!s}{colors[ENDC]}',
-                '    {tcolor} Comment: {comment}{colors[ENDC]}'
+                u'{tcolor}----------{colors[ENDC]}',
+                u'    {tcolor}      ID: {comps[1]}{colors[ENDC]}',
+                u'    {tcolor}Function: {comps[0]}.{comps[3]}{colors[ENDC]}',
+                u'    {tcolor}  Result: {ret[result]!s}{colors[ENDC]}',
+                u'    {tcolor} Comment: {comment}{colors[ENDC]}',
+                u'    {tcolor} Started: {ret[start_time]!s}{colors[ENDC]}',
+                u'    {tcolor}Duration: {ret[duration]!s}{colors[ENDC]}'
             ]
             # This isn't the prettiest way of doing this, but it's readable.
             if comps[1] != comps[2]:
                 state_lines.insert(
-                    3, '    {tcolor}    Name: {comps[2]}{colors[ENDC]}')
+                    3, u'    {tcolor}    Name: {comps[2]}{colors[ENDC]}')
             try:
-                comment = ret['comment'].strip().replace('\n', '\n' + ' ' * 14)
-            except AttributeError:
-                comment = ret['comment'].join(' ').replace('\n',
-                                                           '\n' + ' ' * 13)
+                comment = ret['comment'].strip().replace(
+                    u'\n',
+                    u'\n' + u' ' * 14)
+            except AttributeError:  # Assume comment is a list
+                try:
+                    comment = ret['comment'].join(' ').replace(
+                        u'\n',
+                        u'\n' + u' ' * 13)
+                except AttributeError:
+                    # Comment isn't a list either, just convert to string
+                    comment = str(ret['comment'])
+                    comment = comment.strip().replace(
+                        u'\n',
+                        u'\n' + u' ' * 14)
+            for detail in ['start_time', 'duration']:
+                ret.setdefault(detail, u'')
+            if ret['duration'] != '':
+                ret['duration'] = u'{0} ms'.format(ret['duration'])
             svars = {
                 'tcolor': tcolor,
                 'comps': comps,
@@ -136,39 +225,63 @@ def _format_host(host, data):
                 'colors': colors
             }
             hstrs.extend([sline.format(**svars) for sline in state_lines])
-            changes = '     Changes:   ' + ctext
-            hstrs.append(
-                u'{0}{1}{2[ENDC]}'.format(tcolor, changes, colors)
-            )
+            changes = u'     Changes:   ' + ctext
+            hstrs.append((u'{0}{1}{2[ENDC]}'
+                          .format(tcolor, changes, colors)))
 
         # Append result counts to end of output
-        colorfmt = '{0}{1}{2[ENDC]}'
-        rlabel = {True: 'Succeeded', False: 'Failed', None: 'Not Run'}
+        colorfmt = u'{0}{1}{2[ENDC]}'
+        rlabel = {True: u'Succeeded', False: u'Failed', None: u'Not Run'}
         count_max_len = max([len(str(x)) for x in rcounts.values()] or [0])
         label_max_len = max([len(x) for x in rlabel.values()] or [0])
         line_max_len = label_max_len + count_max_len + 2  # +2 for ': '
         hstrs.append(
             colorfmt.format(
                 colors['CYAN'],
-                '\nSummary\n{0}'.format('-' * line_max_len),
+                u'\nSummary\n{0}'.format('-' * line_max_len),
                 colors
             )
         )
 
         def _counts(label, count):
-            return '{0}: {1:>{2}}'.format(
+            return u'{0}: {1:>{2}}'.format(
                 label,
                 count,
                 line_max_len - (len(label) + 2)
             )
 
         # Successful states
+        changestats = []
+        if None in rcounts and rcounts.get(None, 0) > 0:
+            # test=True states
+            changestats.append(
+                colorfmt.format(
+                    colors['YELLOW'],
+                    u'unchanged={0}'.format(rcounts.get(None, 0)),
+                    colors
+                )
+            )
+        if nchanges > 0:
+            changestats.append(
+                colorfmt.format(
+                    colors['GREEN'],
+                    u'changed={0}'.format(nchanges),
+                    colors
+                )
+            )
+        if changestats:
+            changestats = u' ({0})'.format(', '.join(changestats))
+        else:
+            changestats = u''
         hstrs.append(
             colorfmt.format(
                 colors['GREEN'],
-                _counts(rlabel[True], rcounts.get(True, 0)),
+                _counts(
+                    rlabel[True],
+                    rcounts.get(True, 0) + rcounts.get(None, 0)
+                ),
                 colors
-            )
+            ) + changestats
         )
 
         # Failed states
@@ -181,23 +294,15 @@ def _format_host(host, data):
             )
         )
 
-        # test=True states
-        if None in rcounts:
-            hstrs.append(
-                colorfmt.format(
-                    colors['YELLOW'],
-                    _counts(rlabel[None], rcounts.get(None, 0)),
-                    colors
-                )
-            )
-
-        totals = '{0}\nTotal: {1:>{2}}'.format('-' * line_max_len,
+        totals = u'{0}\nTotal states run: {1:>{2}}'.format('-' * line_max_len,
                                                sum(rcounts.values()),
                                                line_max_len - 7)
         hstrs.append(colorfmt.format(colors['CYAN'], totals, colors))
 
-    hstrs.insert(0, ('{0}{1}:{2[ENDC]}'.format(hcolor, host, colors)))
-    return '\n'.join(hstrs), changed
+    if strip_colors:
+        host = salt.output.strip_esc_sequence(host)
+    hstrs.insert(0, (u'{0}{1}:{2[ENDC]}'.format(hcolor, host, colors)))
+    return u'\n'.join(hstrs), nchanges > 0
 
 
 def _format_changes(changes):
@@ -207,18 +312,18 @@ def _format_changes(changes):
     global __opts__  # pylint: disable=W0601
 
     if not changes:
-        return False, ''
+        return False, u''
 
     if not isinstance(changes, dict):
-        return True, 'Invalid Changes data: {0}'.format(changes)
+        return True, u'Invalid Changes data: {0}'.format(changes)
 
     ret = changes.get('ret')
     if ret is not None and changes.get('out') == 'highstate':
-        ctext = ''
+        ctext = u''
         changed = False
         for host, hostdata in ret.iteritems():
             s, c = _format_host(host, hostdata)
-            ctext += '\n' + '\n'.join((' ' * 14 + l) for l in s.splitlines())
+            ctext += u'\n' + u'\n'.join((u' ' * 14 + l) for l in s.splitlines())
             changed = changed or c
     else:
         changed = True
@@ -226,9 +331,9 @@ def _format_changes(changes):
         # Pass the __opts__ dict. The loader will splat this modules __opts__ dict
         # anyway so have to restore it after the other outputter is done
         if __opts__['color']:
-            __opts__['color'] = 'CYAN'
+            __opts__['color'] = u'CYAN'
         __opts__['nested_indent'] = 14
-        ctext = '\n'
+        ctext = u'\n'
         ctext += salt.output.out_format(
                 changes,
                 'nested',
@@ -244,6 +349,8 @@ def _strip_clean(returns):
     '''
     rm_tags = []
     for tag in returns:
+        if isinstance(tag, dict):
+            continue
         if returns[tag]['result'] and not returns[tag]['changes']:
             rm_tags.append(tag)
     for tag in rm_tags:
@@ -255,19 +362,19 @@ def _format_terse(tcolor, comps, ret, colors, tabular):
     '''
     Terse formatting of a message.
     '''
-    result = "Clean"
+    result = u'Clean'
     if ret['changes']:
-        result = "Changed"
+        result = u'Changed'
     if ret['result'] is False:
-        result = "Failed"
+        result = u'Failed'
     elif ret['result'] is None:
-        result = "Differs"
+        result = u'Differs'
     if tabular is True:
-        fmt_string = '{0}{2:>10}.{3:<10} {4:7}   Name: {1}{5}'
+        fmt_string = u'{0}{2:>10}.{3:<10} {4:7}   Name: {1}{5}'
     elif isinstance(tabular, str):
         fmt_string = tabular
     else:
-        fmt_string = ' {0} Name: {1} - Function: {2}.{3} - Result: {4}{5}'
+        fmt_string = u' {0} Name: {1} - Function: {2}.{3} - Result: {4}{5}'
     msg = fmt_string.format(tcolor,
                             comps[2],
                             comps[0],
