@@ -13,11 +13,18 @@ returner::
       and default_validation_class='UTF8Type';
 
 Required python modules: pycassa
+
+  To use the cassandra returner, append '--return cassandra' to the salt command. ex:
+
+    salt '*' test.ping --return cassandra
 '''
 
 # Import python libs
 import logging
 
+# Import salt libs
+import salt.utils
+
 # Import third party libs
 try:
     import pycassa
@@ -65,3 +72,10 @@ def returner(ret):
 
     log.debug(columns)
     ccf.insert(ret['jid'], columns)
+
+
+def prep_jid(nocache, passed_jid=None):  # pylint: disable=unused-argument
+    '''
+    Do any work necessary to prepare a JID, including sending a custom id
+    '''
+    return passed_jid if passed_jid is not None else salt.utils.gen_jid()
