@@ -14,12 +14,13 @@ import salt.utils.error
 def error(name=None, message=''):
     '''
     If name is None Then return empty dict
-    Otherwise
-       Raise an exception with __name__ from name, message from message
+
+    Otherwise raise an exception with __name__ from name, message from message
 
     CLI Example:
 
     .. code-block:: bash
+
         salt-run error
         salt-run error.error name="Exception" message="This is an error."
     '''
