@@ -49,7 +49,7 @@ def __virtual__():
     '''
     Only load if the mysql module is available
     '''
-    return 'mysql_grants' if 'mysql.grant_exists' in __salt__ else False
+    return 'mysql.grant_exists' in __salt__
 
 
 def _get_mysql_error():
@@ -70,6 +70,7 @@ def present(name,
             grant_option=False,
             escape=True,
             revoke_first=False,
+            ssl_option=False,
             **connection_args):
     '''
     Ensure that the grant is present with the specified properties
@@ -90,10 +91,10 @@ def present(name,
         The network/host that the grant should apply to
 
     grant_option
-        Adds the WITH GRANT OPTION to the defined grant. default: False
+        Adds the WITH GRANT OPTION to the defined grant. Default is ``False``
 
     escape
-        Defines if the database value gets escaped or not. default: True
+        Defines if the database value gets escaped or not. Default is ``True``
 
     revoke_first
         By default, MySQL will not do anything if you issue a command to grant
@@ -110,7 +111,28 @@ def present(name,
         unknown and potentially dangerous state.
         Use with caution!
 
-        default: False
+        Default is ``False``
+
+    ssl_option
+        Adds the specified ssl options for the connecting user as requirements for
+        this grant. Value is a list of single-element dicts corresponding to the
+        list of ssl options to use.
+
+        Possible key/value pairings for the dicts in the value:
+
+        .. code-block:: text
+
+            - SSL: True
+            - X509: True
+            - SUBJECT: <subject>
+            - ISSUER: <issuer>
+            - CIPHER: <cipher>
+
+        The non-boolean ssl options take a string as their values, which should
+        be an appropriate value as specified by the MySQL documentation for these
+        options.
+
+        Default is ``False`` (no ssl options will be used)
     '''
     comment = 'Grant {0} on {1} to {2}@{3} is already present'
     ret = {'name': name,
@@ -161,7 +183,7 @@ def present(name,
         ret['comment'] = ('MySQL grant {0} is set to be created').format(name)
         return ret
     if __salt__['mysql.grant_add'](
-        grant, database, user, host, grant_option, escape, **connection_args
+        grant, database, user, host, grant_option, escape, ssl_option, **connection_args
     ):
         ret['comment'] = 'Grant {0} on {1} to {2}@{3} has been added'
         ret['comment'] = ret['comment'].format(grant, database, user, host)
