@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
     :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
-    :copyright: © 2012-2013 by the SaltStack Team, see AUTHORS for more details
-    :license: Apache 2.0, see LICENSE for more details.
 
 
     tests.unit.utils.event_test
@@ -25,7 +23,7 @@ ensure_in_syspath('../../')
 
 # Import salt libs
 import integration
-from salt.master import clean_proc
+from salt.utils.process import clean_proc
 from salt.utils import event
 
 SOCK_DIR = os.path.join(integration.TMP, 'test-socks')
@@ -58,7 +56,7 @@ class EventSender(Process):
         self.wait = wait
 
     def run(self):
-        me = event.MasterEvent(sock_dir=SOCK_DIR)
+        me = event.MasterEvent(SOCK_DIR)
         time.sleep(self.wait)
         me.fire_event(self.data, self.tag)
         # Wait a few seconds before tearing down the zmq context
@@ -87,9 +85,9 @@ class TestSaltEvent(TestCase):
 
     def assertGotEvent(self, evt, data, msg=None):
         self.assertIsNotNone(evt, msg)
-        for k, v in data.items():
-            self.assertIn(k, evt, msg)
-            self.assertEqual(data[k], evt[k], msg)
+        for key in data:
+            self.assertIn(key, evt, msg)
+            self.assertEqual(data[key], evt[key], msg)
 
     def test_master_event(self):
         me = event.MasterEvent(SOCK_DIR)
@@ -107,8 +105,8 @@ class TestSaltEvent(TestCase):
 
     def test_minion_event(self):
         opts = dict(id='foo', sock_dir=SOCK_DIR)
-        id_hash = hashlib.md5(opts['id']).hexdigest()
-        me = event.MinionEvent(**opts)
+        id_hash = hashlib.md5(opts['id']).hexdigest()[:10]
+        me = event.MinionEvent(opts)
         self.assertEqual(
             me.puburi,
             'ipc://{0}'.format(
@@ -128,13 +126,13 @@ class TestSaltEvent(TestCase):
 
     def test_minion_event_tcp_ipc_mode(self):
         opts = dict(id='foo', ipc_mode='tcp')
-        me = event.MinionEvent(**opts)
+        me = event.MinionEvent(opts)
         self.assertEqual(me.puburi, 'tcp://127.0.0.1:4510')
         self.assertEqual(me.pulluri, 'tcp://127.0.0.1:4511')
 
     def test_minion_event_no_id(self):
-        me = event.MinionEvent(sock_dir=SOCK_DIR)
-        id_hash = hashlib.md5('').hexdigest()
+        me = event.MinionEvent(dict(sock_dir=SOCK_DIR))
+        id_hash = hashlib.md5('').hexdigest()[:10]
         self.assertEqual(
             me.puburi,
             'ipc://{0}'.format(
@@ -155,7 +153,7 @@ class TestSaltEvent(TestCase):
     def test_event_subscription(self):
         '''Test a single event is received'''
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             me.fire_event({'data': 'foo1'}, 'evt1')
             evt1 = me.get_event(tag='evt1')
@@ -164,7 +162,7 @@ class TestSaltEvent(TestCase):
     def test_event_timeout(self):
         '''Test no event is received if the timeout is reached'''
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             me.fire_event({'data': 'foo1'}, 'evt1')
             evt1 = me.get_event(tag='evt1')
@@ -175,7 +173,7 @@ class TestSaltEvent(TestCase):
     def test_event_subscription_matching(self):
         '''Test a subscription startswith matching'''
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             me.fire_event({'data': 'foo1'}, 'evt1')
             evt1 = me.get_event(tag='evt1')
@@ -184,7 +182,7 @@ class TestSaltEvent(TestCase):
     def test_event_subscription_matching_all(self):
         '''Test a subscription matching'''
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             me.fire_event({'data': 'foo1'}, 'evt1')
             evt1 = me.get_event(tag='')
@@ -193,7 +191,7 @@ class TestSaltEvent(TestCase):
     def test_event_not_subscribed(self):
         '''Test get event ignores non-subscribed events'''
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             with eventsender_process({'data': 'foo1'}, 'evt1', 5):
                 me.fire_event({'data': 'foo1'}, 'evt2')
@@ -203,7 +201,7 @@ class TestSaltEvent(TestCase):
     def test_event_multiple_subscriptions(self):
         '''Test multiple subscriptions do not interfere'''
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             with eventsender_process({'data': 'foo1'}, 'evt1', 5):
                 me.fire_event({'data': 'foo1'}, 'evt2')
@@ -213,9 +211,9 @@ class TestSaltEvent(TestCase):
     def test_event_multiple_clients(self):
         '''Test event is received by multiple clients'''
         with eventpublisher_process():
-            me1 = event.MasterEvent(sock_dir=SOCK_DIR)
+            me1 = event.MasterEvent(SOCK_DIR)
             me1.subscribe()
-            me2 = event.MasterEvent(sock_dir=SOCK_DIR)
+            me2 = event.MasterEvent(SOCK_DIR)
             me2.subscribe()
             me1.fire_event({'data': 'foo1'}, 'evt1')
             evt1 = me1.get_event(tag='evt1')
@@ -255,7 +253,7 @@ class TestSaltEvent(TestCase):
         '''Test nested event subscriptions do not drop events, get event for all tags'''
         # Show why not to call get_event(tag='')
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             me.fire_event({'data': 'foo1'}, 'evt1')
             me.fire_event({'data': 'foo2'}, 'evt2')
@@ -267,7 +265,7 @@ class TestSaltEvent(TestCase):
     def test_event_many(self):
         '''Test a large number of events, one at a time'''
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             for i in xrange(500):
                 me.fire_event({'data': '{0}'.format(i)}, 'testevents')
@@ -277,7 +275,7 @@ class TestSaltEvent(TestCase):
     def test_event_many_backlog(self):
         '''Test a large number of events, send all then recv all'''
         with eventpublisher_process():
-            me = event.MasterEvent(sock_dir=SOCK_DIR)
+            me = event.MasterEvent(SOCK_DIR)
             me.subscribe()
             # Must not exceed zmq HWM
             for i in xrange(500):
